#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base

CONF = config.CONF


class ServerConsolesRbacTest(rbac_base.BaseV2ComputeRbacTest):

    @classmethod
    def skip_checks(cls):
        super(ServerConsolesRbacTest, cls).skip_checks()
        if not CONF.compute_feature_enabled.console_output:
            raise cls.skipException('Console output not available.')

    @classmethod
    def resource_setup(cls):
        super(ServerConsolesRbacTest, cls).resource_setup()
        cls.server_id = cls.create_test_server(wait_until='ACTIVE')['id']

    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-console-output")
    @decorators.idempotent_id('90fd80f6-456c-11e7-a919-92ebcb67fe33')
    def test_get_console_output(self):
        with self.rbac_utils.override_role(self):
            self.servers_client.get_console_output(self.server_id)


class ServerConsolesMaxV25RbacTest(rbac_base.BaseV2ComputeRbacTest):

    max_microversion = '2.5'

    @classmethod
    def skip_checks(cls):
        super(ServerConsolesMaxV25RbacTest, cls).skip_checks()
        if not CONF.compute_feature_enabled.console_output:
            raise cls.skipException('Console output not available.')

    @classmethod
    def resource_setup(cls):
        super(ServerConsolesMaxV25RbacTest, cls).resource_setup()
        cls.server_id = cls.create_test_server(wait_until='ACTIVE')['id']

    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-remote-consoles")
    @decorators.idempotent_id('b0a72c02-9b15-4dcb-b186-efe8753370ab')
    def test_get_vnc_console_output(self):
        with self.rbac_utils.override_role(self):
            self.servers_client.get_vnc_console(self.server_id, type="novnc")


class ServerConsolesV26RbacTest(rbac_base.BaseV2ComputeRbacTest):

    min_microversion = '2.6'
    max_microversion = 'latest'

    @classmethod
    def skip_checks(cls):
        super(ServerConsolesV26RbacTest, cls).skip_checks()
        if not CONF.compute_feature_enabled.console_output:
            raise cls.skipException('Console output not available.')

    @classmethod
    def resource_setup(cls):
        super(ServerConsolesV26RbacTest, cls).resource_setup()
        cls.server_id = cls.create_test_server(wait_until='ACTIVE')['id']

    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-remote-consoles")
    @decorators.idempotent_id('879597de-87e0-4da9-a60a-28c8088dc508')
    def test_get_remote_console_output(self):
        with self.rbac_utils.override_role(self):
            self.servers_client.get_remote_console(self.server_id,
                                                   "novnc", "vnc")



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import netaddr

from tempest import config
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.network import rbac_base as base

CONF = config.CONF


class FloatingIpsRbacTest(base.BaseNetworkRbacTest):

    @classmethod
    def resource_setup(cls):
        super(FloatingIpsRbacTest, cls).resource_setup()
        # Create an external network for floating ip creation.
        cls.fip_extnet = cls.create_network(**{'router:external': True})
        # Update router:external attribute to False for proper subnet resource
        # cleanup by base class.
        cls.fip_extnet_id = cls.fip_extnet['id']
        cls.addClassResourceCleanup(
            test_utils.call_and_ignore_notfound_exc,
            cls.networks_client.update_network, cls.fip_extnet_id,
            **{'router:external': False})

        # Create a subnet for the external network
        cls.cidr = netaddr.IPNetwork(CONF.network.project_network_cidr)
        cls.create_subnet(cls.fip_extnet,
                          cidr=cls.cidr,
                          mask_bits=24)

    def _create_floatingip(self, floating_ip_address=None):
        if floating_ip_address is not None:
            body = self.floating_ips_client.create_floatingip(
                floating_network_id=self.fip_extnet_id,
                floating_ip_address=floating_ip_address)
        else:
            body = self.floating_ips_client.create_floatingip(
                floating_network_id=self.fip_extnet_id)

        floating_ip = body['floatingip']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.floating_ips_client.delete_floatingip,
                        floating_ip['id'])

        return floating_ip

    @rbac_rule_validation.action(service="neutron",
                                 rule="create_floatingip")
    @decorators.idempotent_id('f8f7474c-b8a5-4174-af84-73097d6ced38')
    def test_create_floating_ip(self):
        """Create floating IP.

        RBAC test for the neutron create_floatingip policy
        """
        with self.rbac_utils.override_role(self):
            self._create_floatingip()

    @rbac_rule_validation.action(service="neutron",
                                 rule="create_floatingip:floating_ip_address")
    @decorators.idempotent_id('a8bb826a-403d-4130-a55d-120a0a660806')
    def test_create_floating_ip_floatingip_address(self):
        """Create floating IP with address.

        RBAC test for the neutron create_floatingip:floating_ip_address policy
        """
        fip = str(netaddr.IPAddress(self.cidr) + 10)

        with self.rbac_utils.override_role(self):
            self._create_floatingip(floating_ip_address=fip)

    @rbac_rule_validation.action(service="neutron",
                                 rule="update_floatingip")
    @decorators.idempotent_id('2ab1b060-19f8-4ef6-a838-e2ab7b377c63')
    def test_update_floating_ip(self):
        """Update floating IP.

        RBAC test for the neutron update_floatingip policy
        """
        floating_ip = self._create_floatingip()
        with self.rbac_utils.override_role(self):
            # Associate floating IP to the other port
            self.floating_ips_client.update_floatingip(
                floating_ip['id'], port_id=None)

    @rbac_rule_validation.action(service="neutron",
                                 rule="get_floatingip",
                                 expected_error_code=404)
    @decorators.idempotent_id('f8846fd0-c976-48fe-a148-105303931b32')
    def test_show_floating_ip(self):
        """Show floating IP.

        RBAC test for the neutron get_floatingip policy
        """
        floating_ip = self._create_floatingip()
        with self.rbac_utils.override_role(self):
            # Show floating IP
            self.floating_ips_client.show_floatingip(floating_ip['id'])

    @rbac_rule_validation.action(service="neutron",
                                 rule="delete_floatingip",
                                 expected_error_code=404)
    @decorators.idempotent_id('2611b068-30d4-4241-a78f-1b801a14db7e')
    def test_delete_floating_ip(self):
        """Delete floating IP.

        RBAC test for the neutron delete_floatingip policy
        """
        floating_ip = self._create_floatingip()
        with self.rbac_utils.override_role(self):
            # Delete the floating IP
            self.floating_ips_client.delete_floatingip(floating_ip['id'])



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.volume import rbac_base

CONF = config.CONF


class VolumesManageV3RbacTest(rbac_base.BaseVolumeRbacTest):

    @classmethod
    def skip_checks(cls):
        super(VolumesManageV3RbacTest, cls).skip_checks()

        if not CONF.volume_feature_enabled.manage_volume:
            raise cls.skipException("Manage volume tests are disabled")

        if len(CONF.volume.manage_volume_ref) != 2:
            raise cls.skipException("Manage volume ref is not correctly "
                                    "configured")

    @classmethod
    def setup_clients(cls):
        super(VolumesManageV3RbacTest, cls).setup_clients()
        cls.volume_manage_client = cls.os_primary.volume_manage_v2_client

    def _manage_volume(self, org_volume):
        # Manage volume
        new_volume_name = data_utils.rand_name(
            self.__class__.__name__ + '-volume')

        new_volume_ref = {
            'name': new_volume_name,
            'host': org_volume['os-vol-host-attr:host'],
            'ref': {CONF.volume.manage_volume_ref[0]:
                    CONF.volume.manage_volume_ref[1] % org_volume['id']},
            'volume_type': org_volume['volume_type'],
            'availability_zone': org_volume['availability_zone']}

        new_volume_id = self.volume_manage_client.manage_volume(
            **new_volume_ref)['volume']['id']

        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                new_volume_id, 'available')
        self.addCleanup(self.delete_volume,
                        self.volumes_client, new_volume_id)

    @rbac_rule_validation.action(
        service="cinder",
        rule="volume_extension:volume_manage")
    @decorators.idempotent_id('114f9708-939b-407e-aeac-d21ebfabaad3')
    def test_volume_manage(self):
        volume_id = self.create_volume()['id']
        volume = self.volumes_client.show_volume(volume_id)['volume']

        # By default, the volume is managed after creation.  We need to
        # unmanage the volume first before testing manage volume.
        self.volumes_client.unmanage_volume(volume['id'])
        self.volumes_client.wait_for_resource_deletion(volume['id'])

        new_volume_name = data_utils.rand_name(
            self.__class__.__name__ + '-volume')

        new_volume_ref = {
            'name': new_volume_name,
            'host': volume['os-vol-host-attr:host'],
            'ref': {CONF.volume.manage_volume_ref[0]:
                    CONF.volume.manage_volume_ref[1] % volume['id']},
            'volume_type': volume['volume_type'],
            'availability_zone': volume['availability_zone']}

        with self.rbac_utils.override_role(self):
            try:
                new_volume_id = self.volume_manage_client.manage_volume(
                    **new_volume_ref)['volume']['id']
            except exceptions.Forbidden as e:
                # Since the test role under test does not have permission to
                # manage the volume, Forbidden exception is thrown and the
                # manageable list will not be cleaned up. Therefore, we need to
                # re-manage the volume at the end of the test case for proper
                # resource clean up.
                self.addCleanup(self._manage_volume, volume)
                raise exceptions.Forbidden(e)

        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                new_volume_id, 'available')
        self.addCleanup(
            self.delete_volume, self.volumes_client, new_volume_id)

    @rbac_rule_validation.action(
        service="cinder",
        rule="volume_extension:volume_unmanage")
    @decorators.idempotent_id('d5d72abe-60bc-45ac-a8f2-c21b24f0b5d6')
    def test_volume_unmanage(self):
        volume_id = self.create_volume()['id']
        volume = self.volumes_client.show_volume(volume_id)['volume']

        with self.rbac_utils.override_role(self):
            self.volumes_client.unmanage_volume(volume['id'])
        self.volumes_client.wait_for_resource_deletion(volume['id'])

        # In order to clean up the manageable list, we need to re-manage the
        # volume after the test.  The _manage_volume method will set up the
        # proper resource cleanup
        self.addCleanup(self._manage_volume, volume)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.volume import rbac_base


class VolumeTypesExtraSpecsRbacTest(rbac_base.BaseVolumeRbacTest):

    @classmethod
    def skip_checks(cls):
        super(VolumeTypesExtraSpecsRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('os-types-extra-specs', 'volume'):
            msg = "os-types-extra-specs extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(VolumeTypesExtraSpecsRbacTest, cls).resource_setup()
        cls.vol_type = cls.create_volume_type()
        cls.spec_key = data_utils.rand_name(cls.__name__ + '-Spec')

    def _create_volume_type_extra_specs(self, ignore_not_found=False):
        extra_specs = {self.spec_key: "val1"}
        self.volume_types_client.create_volume_type_extra_specs(
            self.vol_type['id'], extra_specs)

        if ignore_not_found:
            self.addCleanup(
                test_utils.call_and_ignore_notfound_exc,
                self.volume_types_client.delete_volume_type_extra_specs,
                self.vol_type['id'], self.spec_key)
        else:
            self.addCleanup(
                self.volume_types_client.delete_volume_type_extra_specs,
                self.vol_type['id'], self.spec_key)

    @decorators.idempotent_id('76c36be2-2b6c-4acf-9aac-c9dc5c17cdbe')
    @rbac_rule_validation.action(
        service="cinder",
        rule="volume_extension:types_extra_specs:index")
    def test_list_volume_types_extra_specs(self):
        with self.rbac_utils.override_role(self):
            self.volume_types_client.list_volume_types_extra_specs(
                self.vol_type['id'])['extra_specs']

    @rbac_rule_validation.action(
        service="cinder",
        rule="volume_extension:types_extra_specs:create")
    @decorators.idempotent_id('eea40251-990b-49b0-99ae-10e4585b479b')
    def test_create_volume_type_extra_specs(self):
        with self.rbac_utils.override_role(self):
            self._create_volume_type_extra_specs(ignore_not_found=True)

    @decorators.idempotent_id('e2dcc9c6-2fef-431d-afaf-92b45bc76d1a')
    @rbac_rule_validation.action(
        service="cinder",
        rule="volume_extension:types_extra_specs:show")
    def test_show_volume_type_extra_specs(self):
        self._create_volume_type_extra_specs()

        with self.rbac_utils.override_role(self):
            self.volume_types_client.show_volume_type_extra_specs(
                self.vol_type['id'], self.spec_key)

    @decorators.idempotent_id('93001912-f938-41c7-8787-62dc7010fd52')
    @rbac_rule_validation.action(
        service="cinder",
        rule="volume_extension:types_extra_specs:delete")
    def test_delete_volume_type_extra_specs(self):
        self._create_volume_type_extra_specs(ignore_not_found=True)

        with self.rbac_utils.override_role(self):
            self.volume_types_client.delete_volume_type_extra_specs(
                self.vol_type['id'], self.spec_key)

    @decorators.idempotent_id('0a444437-7402-4fbe-a18a-93af2ee00618')
    @rbac_rule_validation.action(
        service="cinder",
        rule="volume_extension:types_extra_specs:update")
    def test_update_volume_type_extra_specs(self):
        self._create_volume_type_extra_specs()
        update_extra_specs = {self.spec_key: "val2"}

        with self.rbac_utils.override_role(self):
            self.volume_types_client.update_volume_type_extra_specs(
                self.vol_type['id'], self.spec_key, update_extra_specs)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import netaddr

from tempest.common import utils
from tempest.common.utils import net_utils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_exceptions
from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.network import rbac_base as base

CONF = config.CONF


class PortsRbacTest(base.BaseNetworkRbacTest):

    @classmethod
    def resource_setup(cls):
        super(PortsRbacTest, cls).resource_setup()
        # Create a network and subnet.
        cls.network = cls.create_network()
        cls.cidr = netaddr.IPNetwork(CONF.network.project_network_cidr)
        cls.subnet = cls.create_subnet(cls.network, cidr=cls.cidr,
                                       mask_bits=24)
        cls.ip_range = netaddr.IPRange(
            cls.subnet['allocation_pools'][0]['start'],
            cls.subnet['allocation_pools'][0]['end'])

        cls.port = cls.create_port(cls.network)
        ipaddr = cls.port['fixed_ips'][0]['ip_address']
        cls.port_ip_address = ipaddr
        cls.port_mac_address = cls.port['mac_address']

    def _get_unused_ip_address(self):
        # Pick an unused ip address.
        ip_list = net_utils.get_unused_ip_addresses(self.ports_client,
                                                    self.subnets_client,
                                                    self.network['id'],
                                                    self.subnet['id'],
                                                    1)
        return ip_list

    @rbac_rule_validation.action(service="neutron",
                                 rule="create_port")
    @decorators.idempotent_id('0ec8c551-625c-4864-8a52-85baa7c40f22')
    def test_create_port(self):

        with self.rbac_utils.override_role(self):
            self.create_port(self.network)

    @decorators.idempotent_id('045ee797-4962-4913-b96a-5d7ea04099e7')
    @rbac_rule_validation.action(service="neutron",
                                 rule="create_port:device_owner")
    def test_create_port_device_owner(self):
        with self.rbac_utils.override_role(self):
            self.create_port(self.network,
                             device_owner='network:router_interface')

    @decorators.idempotent_id('c4fa8844-f5ef-4daa-bfa2-b89897dfaedf')
    @rbac_rule_validation.action(service="neutron",
                                 rule="create_port:port_security_enabled")
    def test_create_port_security_enabled(self):
        with self.rbac_utils.override_role(self):
            self.create_port(self.network, port_security_enabled=True)

    @utils.requires_ext(extension='binding', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rule="create_port:binding:host_id")
    @decorators.idempotent_id('a54bd6b8-a7eb-4101-bfe8-093930b0d660')
    def test_create_port_binding_host_id(self):

        post_body = {'network': self.network,
                     'binding:host_id': "rbac_test_host"}

        with self.rbac_utils.override_role(self):
            self.create_port(**post_body)

    @utils.requires_ext(extension='binding', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rule="create_port:binding:profile")
    @decorators.idempotent_id('98fa38ab-c2ed-46a0-99f0-59f18cbd257a')
    def test_create_port_binding_profile(self):

        binding_profile = {"foo": "1"}

        post_body = {'network': self.network,
                     'binding:profile': binding_profile}

        with self.rbac_utils.override_role(self):
            self.create_port(**post_body)

    @rbac_rule_validation.action(service="neutron",
                                 rule="create_port:fixed_ips:ip_address")
    @decorators.idempotent_id('2551e10d-006a-413c-925a-8c6f834c09ac')
    def test_create_port_fixed_ips_ip_address(self):

        ip_list = self._get_unused_ip_address()
        fixed_ips = [{'ip_address': ip_list[0]},
                     {'subnet_id': self.subnet['id']}]

        post_body = {'network': self.network,
                     'fixed_ips': fixed_ips}

        with self.rbac_utils.override_role(self):
            self.create_port(**post_body)

    @rbac_rule_validation.action(service="neutron",
                                 rule="create_port:mac_address")
    @decorators.idempotent_id('aee6d0be-a7f3-452f-aefc-796b4eb9c9a8')
    def test_create_port_mac_address(self):

        post_body = {'network': self.network,
                     'mac_address': data_utils.rand_mac_address()}

        with self.rbac_utils.override_role(self):
            self.create_port(**post_body)

    @rbac_rule_validation.action(service="neutron",
                                 rule="create_port:allowed_address_pairs")
    @decorators.idempotent_id('b638d1f4-d903-4ca8-aa2a-6fd603c5ec3a')
    def test_create_port_allowed_address_pairs(self):

        # Create port with allowed address pair attribute
        allowed_address_pairs = [{'ip_address': self.port_ip_address,
                                  'mac_address': self.port_mac_address}]

        post_body = {'network': self.network,
                     'allowed_address_pairs': allowed_address_pairs}

        with self.rbac_utils.override_role(self):
            self.create_port(**post_body)

    @rbac_rule_validation.action(service="neutron",
                                 rule="get_port",
                                 expected_error_code=404)
    @decorators.idempotent_id('a9d41cb8-78a2-4b97-985c-44e4064416f4')
    def test_show_port(self):
        with self.rbac_utils.override_role(self):
            self.ports_client.show_port(self.port['id'])

    @utils.requires_ext(extension='binding', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rule="get_port:binding:vif_type")
    @decorators.idempotent_id('125aff0b-8fed-4f8e-8410-338616594b06')
    def test_show_port_binding_vif_type(self):

        # Verify specific fields of a port
        fields = ['binding:vif_type']

        with self.rbac_utils.override_role(self):
            retrieved_port = self.ports_client.show_port(
                self.port['id'], fields=fields)['port']

        # Rather than throwing a 403, the field is not present, so raise exc.
        if fields[0] not in retrieved_port:
            raise rbac_exceptions.RbacMalformedResponse(
                attribute='binding:vif_type')

    @utils.requires_ext(extension='binding', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rule="get_port:binding:vif_details")
    @decorators.idempotent_id('e42bfd77-fcce-45ee-9728-3424300f0d6f')
    def test_show_port_binding_vif_details(self):

        # Verify specific fields of a port
        fields = ['binding:vif_details']

        with self.rbac_utils.override_role(self):
            retrieved_port = self.ports_client.show_port(
                self.port['id'], fields=fields)['port']

        # Rather than throwing a 403, the field is not present, so raise exc.
        if fields[0] not in retrieved_port:
            raise rbac_exceptions.RbacMalformedResponse(
                attribute='binding:vif_details')

    @utils.requires_ext(extension='binding', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rule="get_port:binding:host_id")
    @decorators.idempotent_id('8e61bcdc-6f81-443c-833e-44410266551e')
    def test_show_port_binding_host_id(self):

        # Verify specific fields of a port
        fields = ['binding:host_id']
        post_body = {'network': self.network,
                     'binding:host_id': data_utils.rand_name('host-id')}
        port = self.create_port(**post_body)

        with self.rbac_utils.override_role(self):
            retrieved_port = self.ports_client.show_port(
                port['id'], fields=fields)['port']

        # Rather than throwing a 403, the field is not present, so raise exc.
        if fields[0] not in retrieved_port:
            raise rbac_exceptions.RbacMalformedResponse(
                attribute='binding:host_id')

    @utils.requires_ext(extension='binding', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rule="get_port:binding:profile")
    @decorators.idempotent_id('d497cea9-c4ad-42e0-acc9-8d257d6b01fc')
    def test_show_port_binding_profile(self):

        # Verify specific fields of a port
        fields = ['binding:profile']
        binding_profile = {"foo": "1"}
        post_body = {'network': self.network,
                     'binding:profile': binding_profile}
        port = self.create_port(**post_body)

        with self.rbac_utils.override_role(self):
            retrieved_port = self.ports_client.show_port(
                port['id'], fields=fields)['port']

        # Rather than throwing a 403, the field is not present, so raise exc.
        if fields[0] not in retrieved_port:
            raise rbac_exceptions.RbacMalformedResponse(
                attribute='binding:profile')

    @rbac_rule_validation.action(service="neutron",
                                 rule="update_port")
    @decorators.idempotent_id('afa80981-3c59-42fd-9531-3bcb2cd03711')
    def test_update_port(self):
        with self.rbac_utils.override_role(self):
            self.ports_client.update_port(self.port['id'],
                                          admin_state_up=False)
        self.addCleanup(self.ports_client.update_port, self.port['id'],
                        admin_state_up=True)

    @decorators.idempotent_id('08d70f59-67cb-4fb1-bd6c-a5e59dd5db2b')
    @rbac_rule_validation.action(service="neutron",
                                 rule="update_port:device_owner")
    def test_update_port_device_owner(self):
        original_device_owner = self.port['device_owner']

        with self.rbac_utils.override_role(self):
            self.ports_client.update_port(
                self.port['id'], device_owner='network:router_interface')
        self.addCleanup(self.ports_client.update_port, self.port['id'],
                        device_owner=original_device_owner)

    @rbac_rule_validation.action(service="neutron",
                                 rule="update_port:mac_address")
    @decorators.idempotent_id('507140c8-7b14-4d63-b627-2103691d887e')
    def test_update_port_mac_address(self):
        original_mac_address = self.port['mac_address']

        with self.rbac_utils.override_role(self):
            self.ports_client.update_port(
                self.port['id'], mac_address=data_utils.rand_mac_address())
        self.addCleanup(self.ports_client.update_port, self.port['id'],
                        mac_address=original_mac_address)

    @rbac_rule_validation.action(service="neutron",
                                 rule="update_port:fixed_ips:ip_address")
    @decorators.idempotent_id('c091c825-532b-4c6f-a14f-affd3259c1c3')
    def test_update_port_fixed_ips_ip_address(self):

        # Pick an ip address within the allocation_pools range.
        post_body = {'network': self.network}
        port = self.create_port(**post_body)

        ip_list = self._get_unused_ip_address()
        fixed_ips = [{'ip_address': ip_list[0]}]

        with self.rbac_utils.override_role(self):
            self.ports_client.update_port(port['id'], fixed_ips=fixed_ips)

    @rbac_rule_validation.action(service="neutron",
                                 rule="update_port:port_security_enabled")
    @decorators.idempotent_id('795541af-6652-4e35-9581-fd58224f7545')
    def test_update_port_security_enabled(self):
        with self.rbac_utils.override_role(self):
            self.ports_client.update_port(self.port['id'], security_groups=[])

    @utils.requires_ext(extension='binding', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rule="update_port:binding:host_id")
    @decorators.idempotent_id('24206a72-0d90-4712-918c-5c9a1ebef64d')
    def test_update_port_binding_host_id(self):

        post_body = {'network': self.network,
                     'binding:host_id': 'rbac_test_host'}
        port = self.create_port(**post_body)

        updated_body = {'port_id': port['id'],
                        'binding:host_id': 'rbac_test_host_updated'}

        with self.rbac_utils.override_role(self):
            self.ports_client.update_port(**updated_body)

    @utils.requires_ext(extension='binding', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rule="update_port:binding:profile")
    @decorators.idempotent_id('990ea8d1-9257-4f71-a3bf-d6d0914625c5')
    def test_update_port_binding_profile(self):

        binding_profile = {"foo": "1"}
        post_body = {'network': self.network,
                     'binding:profile': binding_profile}

        port = self.create_port(**post_body)

        new_binding_profile = {"foo": "2"}
        updated_body = {'port_id': port['id'],
                        'binding:profile': new_binding_profile}

        with self.rbac_utils.override_role(self):
            self.ports_client.update_port(**updated_body)

    @rbac_rule_validation.action(service="neutron",
                                 rule="update_port:allowed_address_pairs")
    @decorators.idempotent_id('729c2151-bb49-4f4f-9d58-3ed8819b7582')
    def test_update_port_allowed_address_pairs(self):

        ip_list = self._get_unused_ip_address()
        # Update allowed address pair attribute of port
        address_pairs = [{'ip_address': ip_list[0],
                          'mac_address': data_utils.rand_mac_address()}]
        post_body = {'network': self.network}
        port = self.create_port(**post_body)

        with self.rbac_utils.override_role(self):
            self.ports_client.update_port(port['id'],
                                          allowed_address_pairs=address_pairs)

    @rbac_rule_validation.action(service="neutron",
                                 rule="delete_port",
                                 expected_error_code=404)
    @decorators.idempotent_id('1cf8e582-bc09-46cb-b32a-82bf991ad56f')
    def test_delete_port(self):

        port = self.create_port(self.network)
        with self.rbac_utils.override_role(self):
            self.ports_client.delete_port(port['id'])



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils

from tempest import config
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.identity import rbac_base

CONF = config.CONF


class IdentityOAuthTokensV3RbacTest(rbac_base.BaseIdentityV3RbacTest):

    @classmethod
    def resource_setup(cls):
        super(IdentityOAuthTokensV3RbacTest, cls).resource_setup()
        # Authorize token on admin role since primary user has admin
        # credentials before switching roles. Populate role_ids with admin
        # role id.
        cls.role_ids = [cls.get_role_by_name(CONF.identity.admin_role)['id']]
        cls.project_id = cls.os_primary.credentials.project_id
        cls.user_id = cls.os_primary.credentials.user_id

    def _create_consumer(self):
        description = data_utils.rand_name(
            self.__class__.__name__ + '-Consumer')
        consumer = self.consumers_client.create_consumer(
            description)['consumer']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.consumers_client.delete_consumer,
                        consumer['id'])
        return consumer

    def _create_consumer_and_request_token(self):
        # Create consumer
        consumer = self._create_consumer()

        # Create request token
        request_token = self.oauth_token_client.create_request_token(
            consumer['id'], consumer['secret'], self.project_id)

        return consumer, request_token

    def _create_access_token(self):
        consumer, request_token = self._create_consumer_and_request_token()

        # Authorize request token
        resp = self.oauth_token_client.authorize_request_token(
            request_token['oauth_token'], self.role_ids)['token']
        auth_verifier = resp['oauth_verifier']

        # Create access token
        body = self.oauth_token_client.create_access_token(
            consumer['id'],
            consumer['secret'],
            request_token['oauth_token'],
            request_token['oauth_token_secret'],
            auth_verifier)
        access_key = body['oauth_token']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.oauth_token_client.revoke_access_token,
                        self.user_id, access_key)

        return access_key

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:authorize_request_token")
    @decorators.idempotent_id('0f148510-63bf-11e6-4522-080044d0d976')
    def test_authorize_request_token(self):
        _, request_token = self._create_consumer_and_request_token()

        with self.rbac_utils.override_role(self):
            self.oauth_token_client.authorize_request_token(
                request_token['oauth_token'],
                self.role_ids)

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:get_access_token")
    @decorators.idempotent_id('0f148510-63bf-11e6-4522-080044d0d977')
    def test_get_access_token(self):
        access_token = self._create_access_token()

        with self.rbac_utils.override_role(self):
            self.oauth_token_client.get_access_token(self.user_id,
                                                     access_token)

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:get_access_token_role")
    @decorators.idempotent_id('0f148510-63bf-11e6-4522-080044d0d980')
    def test_get_access_token_role(self):
        access_token = self._create_access_token()

        with self.rbac_utils.override_role(self):
            self.oauth_token_client.get_access_token_role(
                self.user_id, access_token, self.role_ids[0])

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:list_access_tokens")
    @decorators.idempotent_id('0f148510-63bf-11e6-4522-080044d0d979')
    def test_list_access_tokens(self):
        with self.rbac_utils.override_role(self):
            self.oauth_token_client.list_access_tokens(self.user_id)

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:list_access_token_roles")
    @decorators.idempotent_id('0f148510-63bf-11e6-4522-080044d0d978')
    def test_list_access_token_roles(self):
        access_token = self._create_access_token()

        with self.rbac_utils.override_role(self):
            self.oauth_token_client.list_access_token_roles(
                self.user_id, access_token)

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:delete_access_token")
    @decorators.idempotent_id('0f148510-63bf-11e6-4522-080044d0d981')
    def test_revoke_access_token(self):
        access_token = self._create_access_token()

        with self.rbac_utils.override_role(self):
            self.oauth_token_client.revoke_access_token(
                self.user_id, access_token)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc
from contextlib import contextmanager
import debtcollector.removals
import six
import time

from oslo_log import log as logging
from oslo_utils import excutils

from tempest import clients
from tempest.common import credentials_factory as credentials
from tempest import config

from patrole_tempest_plugin import rbac_exceptions

CONF = config.CONF
LOG = logging.getLogger(__name__)


class RbacUtils(object):
    """Utility class responsible for switching ``os_primary`` role.

    This class is responsible for overriding the value of the primary Tempest
    credential's role (i.e. ``os_primary`` role). By doing so, it is possible
    to seamlessly swap between admin credentials, needed for setup and clean
    up, and primary credentials, needed to perform the API call which does
    policy enforcement. The primary credentials always cycle between roles
    defined by ``CONF.identity.admin_role`` and
    ``CONF.patrole.rbac_test_role``.
    """

    def __init__(self, test_obj):
        """Constructor for ``RbacUtils``.

        :param test_obj: An instance of `tempest.test.BaseTestCase`.
        """
        # Intialize the admin roles_client to perform role switching.
        admin_mgr = clients.Manager(
            credentials.get_configured_admin_credentials())
        if test_obj.get_identity_version() == 'v3':
            admin_roles_client = admin_mgr.roles_v3_client
        else:
            admin_roles_client = admin_mgr.roles_client

        self.admin_roles_client = admin_roles_client
        self._override_role(test_obj, False)

    admin_role_id = None
    rbac_role_id = None

    @contextmanager
    def override_role(self, test_obj):
        """Override the role used by ``os_primary`` Tempest credentials.

        Temporarily change the role used by ``os_primary`` credentials to:

        * ``[patrole] rbac_test_role`` before test execution
        * ``[identity] admin_role`` after test execution

        Automatically switches to admin role after test execution.

        :param test_obj: Instance of ``tempest.test.BaseTestCase``.
        :returns: None

        .. warning::

            This function can alter user roles for pre-provisioned credentials.
            Work is underway to safely clean up after this function.

        Example::

            @rbac_rule_validation.action(service='test',
                                         rule='a:test:rule')
            def test_foo(self):
                # Allocate test-level resources here.
                with self.rbac_utils.override_role(self):
                    # The role for `os_primary` has now been overriden. Within
                    # this block, call the API endpoint that enforces the
                    # expected policy specified by "rule" in the decorator.
                    self.foo_service.bar_api_call()
                # The role is switched back to admin automatically. Note that
                # if the API call above threw an exception, any code below this
                # point in the test is not executed.
        """
        self._override_role(test_obj, True)
        try:
            # Execute the test.
            yield
        finally:
            # This code block is always executed, no matter the result of the
            # test. Automatically switch back to the admin role for test clean
            # up.
            self._override_role(test_obj, False)

    @debtcollector.removals.remove(removal_version='Rocky')
    def switch_role(self, test_obj, toggle_rbac_role):
        """Switch the role used by `os_primary` Tempest credentials.

        Switch the role used by `os_primary` credentials to:

        * admin if `toggle_rbac_role` is False
        * `CONF.patrole.rbac_test_role` if `toggle_rbac_role` is True

        :param test_obj: instance of :py:class:`tempest.test.BaseTestCase`
        :param toggle_rbac_role: role to switch `os_primary` Tempest creds to
        """
        self._override_role(test_obj, toggle_rbac_role)

    def _override_role(self, test_obj, toggle_rbac_role=False):
        """Private helper for overriding ``os_primary`` Tempest credentials.

        :param test_obj: instance of :py:class:`tempest.test.BaseTestCase`
        :param toggle_rbac_role: Boolean value that controls the role that
            overrides default role of ``os_primary`` credentials.
            * If True: role is set to ``[patrole] rbac_test_role``
            * If False: role is set to ``[identity] admin_role``
        """
        self.user_id = test_obj.os_primary.credentials.user_id
        self.project_id = test_obj.os_primary.credentials.tenant_id
        self.token = test_obj.os_primary.auth_provider.get_token()

        LOG.debug('Overriding role to: %s.', toggle_rbac_role)
        role_already_present = False

        try:
            if not all([self.admin_role_id, self.rbac_role_id]):
                self._get_roles_by_name()

            target_role = (
                self.rbac_role_id if toggle_rbac_role else self.admin_role_id)
            role_already_present = self._list_and_clear_user_roles_on_project(
                target_role)

            # Do not override roles if `target_role` already exists.
            if not role_already_present:
                self._create_user_role_on_project(target_role)
        except Exception as exp:
            with excutils.save_and_reraise_exception():
                LOG.exception(exp)
        finally:
            test_obj.os_primary.auth_provider.clear_auth()
            # Fernet tokens are not subsecond aware so sleep to ensure we are
            # passing the second boundary before attempting to authenticate.
            # Only sleep if a token revocation occurred as a result of role
            # overriding. This will optimize test runtime in the case where
            # ``[identity] admin_role`` == ``[patrole] rbac_test_role``.
            if not role_already_present:
                time.sleep(1)
            test_obj.os_primary.auth_provider.set_auth()

    def _get_roles_by_name(self):
        available_roles = self.admin_roles_client.list_roles()
        admin_role_id = rbac_role_id = None

        for role in available_roles['roles']:
            if role['name'] == CONF.patrole.rbac_test_role:
                rbac_role_id = role['id']
            if role['name'] == CONF.identity.admin_role:
                admin_role_id = role['id']

        if not all([admin_role_id, rbac_role_id]):
            msg = ("Roles defined by `[patrole] rbac_test_role` and "
                   "`[identity] admin_role` must be defined in the system.")
            raise rbac_exceptions.RbacResourceSetupFailed(msg)

        self.admin_role_id = admin_role_id
        self.rbac_role_id = rbac_role_id

    def _create_user_role_on_project(self, role_id):
        self.admin_roles_client.create_user_role_on_project(
            self.project_id, self.user_id, role_id)

    def _list_and_clear_user_roles_on_project(self, role_id):
        roles = self.admin_roles_client.list_user_roles_on_project(
            self.project_id, self.user_id)['roles']
        role_ids = [role['id'] for role in roles]

        # NOTE(felipemonteiro): We do not use ``role_id in role_ids`` here to
        # avoid over-permission errors: if the current list of roles on the
        # project includes "admin" and "Member", and we are switching to the
        # "Member" role, then we must delete the "admin" role. Thus, we only
        # return early if the user's roles on the project are an exact match.
        if [role_id] == role_ids:
            return True

        for role in roles:
            self.admin_roles_client.delete_role_from_user_on_project(
                self.project_id, self.user_id, role['id'])

        return False


class RbacUtilsMixin(object):
    """Mixin class to be used alongside an instance of
    :py:class:`tempest.test.BaseTestCase`.

    Should be used to perform Patrole class setup for a base RBAC class. Child
    classes should not use this mixin.

    Example::

        class BaseRbacTest(rbac_utils.RbacUtilsMixin, base.BaseV2ComputeTest):

            @classmethod
            def skip_checks(cls):
                super(BaseRbacTest, cls).skip_checks()
                cls.skip_rbac_checks()

            @classmethod
            def setup_clients(cls):
                super(BaseRbacTest, cls).setup_clients()
                cls.setup_rbac_utils()
    """

    @classmethod
    def skip_rbac_checks(cls):
        if not CONF.patrole.enable_rbac:
            raise cls.skipException(
                '%s skipped as Patrole testing not enabled.' % cls.__name__)

    @classmethod
    def setup_rbac_utils(cls):
        cls.rbac_utils = RbacUtils(cls)


def is_admin():
    """Verifies whether the current test role equals the admin role.

    :returns: True if ``rbac_test_role`` is the admin role.
    """
    return CONF.patrole.rbac_test_role == CONF.identity.admin_role


@six.add_metaclass(abc.ABCMeta)
class RbacAuthority(object):
    """Class for validating whether a given role can perform a policy action.

    Any class that extends ``RbacAuthority`` provides the logic for determining
    whether a role has permissions to execute a policy action.
    """

    @abc.abstractmethod
    def allowed(self, rule, role):
        """Determine whether the role should be able to perform the API.

        :param rule: The name of the policy enforced by the API.
        :param role: The role used to determine whether ``rule`` can be
            executed.
        :returns: True if the ``role`` has permissions to execute
            ``rule``, else False.
        """



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log
from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_exceptions
from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.network import rbac_base as base

LOG = log.getLogger(__name__)


class NetworksMultiProviderRbacTest(base.BaseNetworkRbacTest):

    @classmethod
    def skip_checks(cls):
        super(NetworksMultiProviderRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('multi-provider', 'network'):
            msg = "multi-provider extension not enabled."
            raise cls.skipException(msg)

    def _create_network_segments(self):
        segments = [{"provider:network_type": "gre"},
                    {"provider:network_type": "gre"}]

        body = self.networks_client.create_network(
            name=data_utils.rand_name(self.__class__.__name__),
            segments=segments)
        network = body['network']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.networks_client.delete_network,
                        network['id'])
        return network

    @rbac_rule_validation.action(service="neutron",
                                 rule="create_network:segments")
    @decorators.idempotent_id('9e1d0c3d-92e3-40e3-855e-bfbb72ea6e0b')
    def test_create_network_segments(self):
        """Create network with segments.

        RBAC test for the neutron create_network:segments policy
        """
        with self.rbac_utils.override_role(self):
            self._create_network_segments()

    @rbac_rule_validation.action(service="neutron",
                                 rule="update_network:segments")
    @decorators.idempotent_id('0f45232a-7b59-4bb1-9a91-db77d0a8cc9b')
    def test_update_network_segments(self):
        """Update network segments.

        RBAC test for the neutron update_network:segments policy
        """
        network = self._create_network_segments()
        new_segments = [{"provider:network_type": "gre"}]

        with self.rbac_utils.override_role(self):
            self.networks_client.update_network(network['id'],
                                                segments=new_segments)

    @rbac_rule_validation.action(service="neutron",
                                 rule="get_network:segments")
    @decorators.idempotent_id('094ff9b7-0c3b-4515-b19b-b9d2031337bd')
    def test_show_network_segments(self):
        """Show network segments.

        RBAC test for the neutron get_network:segments policy
        """
        network = self._create_network_segments()

        with self.rbac_utils.override_role(self):
            body = self.networks_client.show_network(network['id'],
                                                     fields='segments')
        response_network = body['network']

        # If user does not have access to the network segments attribute,
        # no NotFound or Forbidden exception are thrown.  Instead,
        # the response will have an empty network body only.
        if len(response_network) == 0:
            LOG.info("NotFound or Forbidden exception are not thrown when "
                     "role doesn't have access to the endpoint. Instead, "
                     "the response will have an empty network body.")
            raise rbac_exceptions.RbacMalformedResponse(True)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
import testtools

from tempest.lib import exceptions as lib_exc
from tempest import test
from tempest.tests import base

from patrole_tempest_plugin import rbac_exceptions
from patrole_tempest_plugin import rbac_utils
from patrole_tempest_plugin.tests.unit import fixtures as patrole_fixtures


class RBACUtilsTest(base.TestCase):

    def setUp(self):
        super(RBACUtilsTest, self).setUp()
        # Reset the role history after each test run to avoid validation
        # errors between tests.
        rbac_utils.RbacUtils.override_role_history = {}
        self.rbac_utils = self.useFixture(patrole_fixtures.RbacUtilsFixture())

    def test_override_role_with_missing_admin_role(self):
        self.rbac_utils.set_roles('member')
        error_re = (
            'Roles defined by `\[patrole\] rbac_test_role` and `\[identity\] '
            'admin_role` must be defined in the system.')
        self.assertRaisesRegex(rbac_exceptions.RbacResourceSetupFailed,
                               error_re, self.rbac_utils.override_role)

    def test_override_role_with_missing_rbac_role(self):
        self.rbac_utils.set_roles('admin')
        error_re = (
            'Roles defined by `\[patrole\] rbac_test_role` and `\[identity\] '
            'admin_role` must be defined in the system.')
        self.assertRaisesRegex(rbac_exceptions.RbacResourceSetupFailed,
                               error_re, self.rbac_utils.override_role)

    def test_override_role_to_admin_role(self):
        self.rbac_utils.override_role()

        mock_test_obj = self.rbac_utils.mock_test_obj
        roles_client = self.rbac_utils.roles_v3_client
        mock_time = self.rbac_utils.mock_time

        roles_client.create_user_role_on_project.assert_called_once_with(
            self.rbac_utils.PROJECT_ID, self.rbac_utils.USER_ID, 'admin_id')
        mock_test_obj.os_primary.auth_provider.clear_auth\
            .assert_called_once_with()
        mock_test_obj.os_primary.auth_provider.set_auth\
            .assert_called_once_with()
        mock_time.sleep.assert_called_once_with(1)

    def test_override_role_to_admin_role_avoids_role_switch(self):
        self.rbac_utils.set_roles(['admin', 'member'], 'admin')
        self.rbac_utils.override_role()

        roles_client = self.rbac_utils.roles_v3_client
        mock_time = self.rbac_utils.mock_time

        roles_client.create_user_role_on_project.assert_not_called()
        mock_time.sleep.assert_not_called()

    def test_override_role_to_member_role(self):
        self.rbac_utils.override_role(True)

        mock_test_obj = self.rbac_utils.mock_test_obj
        roles_client = self.rbac_utils.roles_v3_client
        mock_time = self.rbac_utils.mock_time

        roles_client.create_user_role_on_project.assert_has_calls([
            mock.call(self.rbac_utils.PROJECT_ID, self.rbac_utils.USER_ID,
                      'admin_id'),
            mock.call(self.rbac_utils.PROJECT_ID, self.rbac_utils.USER_ID,
                      'member_id')
        ])
        mock_test_obj.os_primary.auth_provider.clear_auth.assert_has_calls(
            [mock.call()] * 2)
        mock_test_obj.os_primary.auth_provider.set_auth.assert_has_calls(
            [mock.call()] * 2)
        mock_time.sleep.assert_has_calls([mock.call(1)] * 2)

    def test_override_role_to_member_role_avoids_role_switch(self):
        self.rbac_utils.set_roles(['admin', 'member'], 'member')
        self.rbac_utils.override_role(True)

        roles_client = self.rbac_utils.roles_v3_client
        mock_time = self.rbac_utils.mock_time

        roles_client.create_user_role_on_project.assert_has_calls([
            mock.call(self.rbac_utils.PROJECT_ID, self.rbac_utils.USER_ID,
                      'admin_id')
        ])
        mock_time.sleep.assert_called_once_with(1)

    def test_override_role_to_member_role_then_admin_role(self):
        self.rbac_utils.override_role(True, False)

        mock_test_obj = self.rbac_utils.mock_test_obj
        roles_client = self.rbac_utils.roles_v3_client
        mock_time = self.rbac_utils.mock_time

        roles_client.create_user_role_on_project.assert_has_calls([
            mock.call(self.rbac_utils.PROJECT_ID, self.rbac_utils.USER_ID,
                      'admin_id'),
            mock.call(self.rbac_utils.PROJECT_ID, self.rbac_utils.USER_ID,
                      'member_id'),
            mock.call(self.rbac_utils.PROJECT_ID, self.rbac_utils.USER_ID,
                      'admin_id')
        ])
        mock_test_obj.os_primary.auth_provider.clear_auth.assert_has_calls(
            [mock.call()] * 3)
        mock_test_obj.os_primary.auth_provider.set_auth.assert_has_calls(
            [mock.call()] * 3)
        mock_time.sleep.assert_has_calls([mock.call(1)] * 3)

    def test_clear_user_roles(self):
        # NOTE(felipemonteiro): Set the user's roles on the project to
        # include 'random' to coerce a role switch, or else it will be
        # skipped.
        self.rbac_utils.set_roles(['admin', 'member'], ['member', 'random'])
        self.rbac_utils.override_role()

        roles_client = self.rbac_utils.roles_v3_client

        roles_client.list_user_roles_on_project.assert_called_once_with(
            self.rbac_utils.PROJECT_ID, self.rbac_utils.USER_ID)
        roles_client.delete_role_from_user_on_project.\
            assert_has_calls([
                mock.call(mock.sentinel.project_id, mock.sentinel.user_id,
                          'member_id'),
                mock.call(mock.sentinel.project_id, mock.sentinel.user_id,
                          'random_id')])

    @mock.patch.object(rbac_utils.RbacUtils, '_override_role', autospec=True)
    def test_override_role_context_manager_simulate_pass(self,
                                                         mock_override_role):
        """Validate that expected override_role calls are made when switching
        to admin role for success path.
        """
        test_obj = mock.MagicMock()
        _rbac_utils = rbac_utils.RbacUtils(test_obj)

        # Validate constructor called _override_role with False.
        mock_override_role.assert_called_once_with(_rbac_utils, test_obj,
                                                   False)
        mock_override_role.reset_mock()

        with _rbac_utils.override_role(test_obj):
            # Validate `override_role` public method called private method
            # `_override_role` with True.
            mock_override_role.assert_called_once_with(_rbac_utils, test_obj,
                                                       True)
            mock_override_role.reset_mock()
        # Validate that `override_role` switched back to admin role after
        # contextmanager.
        mock_override_role.assert_called_once_with(_rbac_utils, test_obj,
                                                   False)

    @mock.patch.object(rbac_utils.RbacUtils, '_override_role', autospec=True)
    def test_override_role_context_manager_simulate_fail(self,
                                                         mock_override_role):
        """Validate that expected override_role calls are made when switching
        to admin role for failure path (i.e. when test raises exception).
        """
        test_obj = mock.MagicMock()
        _rbac_utils = rbac_utils.RbacUtils(test_obj)

        # Validate constructor called _override_role with False.
        mock_override_role.assert_called_once_with(_rbac_utils, test_obj,
                                                   False)
        mock_override_role.reset_mock()

        def _do_test():
            with _rbac_utils.override_role(test_obj):
                # Validate `override_role` public method called private method
                # `_override_role` with True.
                mock_override_role.assert_called_once_with(
                    _rbac_utils, test_obj, True)
                mock_override_role.reset_mock()
                # Raise exc to verify role switch works for negative case.
                raise lib_exc.Forbidden()

        # Validate that role is switched back to admin, despite test failure.
        with testtools.ExpectedException(lib_exc.Forbidden):
            _do_test()
        mock_override_role.assert_called_once_with(_rbac_utils, test_obj,
                                                   False)


class RBACUtilsMixinTest(base.TestCase):

    def setUp(self):
        super(RBACUtilsMixinTest, self).setUp()

        class FakeRbacTest(rbac_utils.RbacUtilsMixin, test.BaseTestCase):

            @classmethod
            def skip_checks(cls):
                super(FakeRbacTest, cls).skip_checks()
                cls.skip_rbac_checks()

            @classmethod
            def setup_clients(cls):
                super(FakeRbacTest, cls).setup_clients()
                cls.setup_rbac_utils()

            def runTest(self):
                pass

        self.parent_class = FakeRbacTest

    def test_setup_rbac_utils(self):
        """Validate that the child class has the `rbac_utils` attribute after
        running parent class's `cls.setup_rbac_utils`.
        """
        class ChildRbacTest(self.parent_class):
            pass

        child_test = ChildRbacTest()

        with mock.patch.object(rbac_utils.RbacUtils, '__init__',
                               lambda *args: None):
            child_test.setUpClass()

        self.assertTrue(hasattr(child_test, 'rbac_utils'))
        self.assertIsInstance(child_test.rbac_utils, rbac_utils.RbacUtils)

    def test_skip_rbac_checks(self):
        """Validate that the child class is skipped if `[patrole] enable_rbac`
        is False and that the child class's name is in the skip message.
        """
        self.useFixture(patrole_fixtures.ConfPatcher(enable_rbac=False,
                                                     group='patrole'))

        class ChildRbacTest(self.parent_class):
            pass

        child_test = ChildRbacTest()

        with testtools.ExpectedException(
                testtools.TestCase.skipException,
                value_re=('%s skipped as Patrole testing not enabled.'
                          % ChildRbacTest.__name__)):
            child_test.setUpClass()



# Copyright 2017 NEC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.common import waiters
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.volume import rbac_base


class BaseGroupSnapshotsRbacTest(rbac_base.BaseVolumeRbacTest):

    def _create_group_snapshot(self, **kwargs):
        if 'name' not in kwargs:
            kwargs['name'] = data_utils.rand_name(
                self.__class__.__name__ + '-Group_Snapshot')
        group_snapshot = self.group_snapshots_client.create_group_snapshot(
            **kwargs)['group_snapshot']
        group_snapshot['group_id'] = kwargs['group_id']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self._delete_group_snapshot, group_snapshot)
        waiters.wait_for_volume_resource_status(
            self.group_snapshots_client, group_snapshot['id'], 'available')
        snapshots = self.snapshots_client.list_snapshots(
            detail=True)['snapshots']
        for snap in snapshots:
            if self.vol['id'] == snap['volume_id']:
                waiters.wait_for_volume_resource_status(
                    self.snapshots_client, snap['id'], 'available')
        return group_snapshot

    def _delete_group_snapshot(self, group_snapshot):
        self.group_snapshots_client.delete_group_snapshot(group_snapshot['id'])
        vols = self.volumes_client.list_volumes(detail=True)['volumes']
        snapshots = self.snapshots_client.list_snapshots(
            detail=True)['snapshots']
        for vol in vols:
            for snap in snapshots:
                if (vol['group_id'] == group_snapshot['group_id'] and
                        vol['id'] == snap['volume_id']):
                    self.snapshots_client.wait_for_resource_deletion(
                        snap['id'])
        self.group_snapshots_client.wait_for_resource_deletion(
            group_snapshot['id'])


class GroupSnaphotsV314RbacTest(BaseGroupSnapshotsRbacTest):
    _api_version = 3
    min_microversion = '3.14'
    max_microversion = 'latest'

    @classmethod
    def skip_checks(cls):
        super(GroupSnaphotsV314RbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('groupsnapshot', 'volume'):
            msg = "%s skipped as group snapshots not enabled." % cls.__name__
            raise cls.skipException(msg)

    @classmethod
    def setup_clients(cls):
        super(GroupSnaphotsV314RbacTest, cls).setup_clients()
        cls.group_snapshot_client = \
            cls.os_primary.group_snapshots_v3_client

    def setUp(self):
        super(GroupSnaphotsV314RbacTest, self).setUp()
        self.volume_type = self.create_volume_type()
        self.group_type = self.create_group_type()
        self.grp = self.create_group(group_type=self.group_type['id'],
                                     volume_types=[self.volume_type['id']])
        self.vol = self.create_volume(volume_type=self.volume_type['id'],
                                      group_id=self.grp['id'])

    @decorators.idempotent_id('653df0e8-d90a-474a-a5ce-3c2339aff7ba')
    @rbac_rule_validation.action(
        service="cinder",
        rule="group:create_group_snapshot"
    )
    def test_create_group_snapshot(self):
        with self.rbac_utils.override_role(self):
            name = data_utils.rand_name(
                self.__class__.__name__ + '-Group_Snapshot')
            group_snapshot = self.group_snapshots_client.create_group_snapshot(
                name=name, group_id=self.grp['id'])['group_snapshot']
        group_snapshot['group_id'] = self.grp['id']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self._delete_group_snapshot, group_snapshot)
        waiters.wait_for_volume_resource_status(
            self.group_snapshots_client, group_snapshot['id'], 'available')
        snapshots = self.snapshots_client.list_snapshots(
            detail=True)['snapshots']
        for snap in snapshots:
            if self.vol['id'] == snap['volume_id']:
                waiters.wait_for_volume_resource_status(
                    self.snapshots_client, snap['id'], 'available')

    @decorators.idempotent_id('8b966844-4421-4f73-940b-9157cb878331')
    @rbac_rule_validation.action(
        service="cinder",
        rule="group:get_group_snapshot"
    )
    def test_show_group_snapshot_rbac(self):
        group_snapshot_name = data_utils.rand_name('group_snapshot')
        group_snapshot = self._create_group_snapshot(group_id=self.grp['id'],
                                                     name=group_snapshot_name)
        with self.rbac_utils.override_role(self):
            self.group_snapshots_client.show_group_snapshot(
                group_snapshot['id'])

    @decorators.idempotent_id('e9de6dae-1efb-47cd-a3a8-d1f4b8f9f3ff')
    @rbac_rule_validation.action(
        service="cinder",
        rule="group:get_all_group_snapshots"
    )
    def test_list_group_snapshot_rbac(self):
        with self.rbac_utils.override_role(self):
            self.group_snapshots_client.list_group_snapshots()

    @decorators.idempotent_id('cf2e25ee-ca58-4ad6-b98d-33235c77db7b')
    @rbac_rule_validation.action(
        service="cinder",
        rule="group:delete_group_snapshot"
        )
    def test_delete_group_snapshot_rbac(self):
        group_snapshot_name = data_utils.rand_name('group_snapshot')
        group_snapshot = self._create_group_snapshot(group_id=self.grp['id'],
                                                     name=group_snapshot_name)
        with self.rbac_utils.override_role(self):
            self.group_snapshots_client.delete_group_snapshot(
                group_snapshot['id'])
        vols = self.volumes_client.list_volumes(detail=True)['volumes']
        snapshots = self.snapshots_client.list_snapshots(
            detail=True)['snapshots']
        for vol in vols:
            for snap in snapshots:
                if (vol['group_id'] == group_snapshot['group_id'] and
                        vol['id'] == snap['volume_id']):
                    self.snapshots_client.wait_for_resource_deletion(
                        snap['id'])
        self.group_snapshots_client.wait_for_resource_deletion(
            group_snapshot['id'])


class GroupSnaphotsV319RbacTest(BaseGroupSnapshotsRbacTest):
    _api_version = 3
    min_microversion = '3.19'
    max_microversion = 'latest'

    @classmethod
    def skip_checks(cls):
        super(GroupSnaphotsV319RbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('groupsnapshot', 'volume'):
            msg = "%s skipped as group snapshots not enabled." % cls.__name__
            raise cls.skipException(msg)

    @classmethod
    def setup_clients(cls):
        super(GroupSnaphotsV319RbacTest, cls).setup_clients()
        cls.group_snapshot_client = \
            cls.os_primary.group_snapshots_v3_client

    def setUp(self):
        super(GroupSnaphotsV319RbacTest, self).setUp()
        self.volume_type = self.create_volume_type()
        self.group_type = self.create_group_type()
        self.grp = self.create_group(group_type=self.group_type['id'],
                                     volume_types=[self.volume_type['id']])
        self.vol = self.create_volume(volume_type=self.volume_type['id'],
                                      group_id=self.grp['id'])

    @decorators.idempotent_id('3f0c842e-0c72-4f5e-a9c2-281070be3e2c')
    @rbac_rule_validation.action(
        service="cinder",
        rule="group:reset_group_snapshot_status"
        )
    def test_reset_group_snapshot_rbac(self):
        group_snapshot_name = data_utils.rand_name('group_snapshot')
        group_snapshot = self._create_group_snapshot(group_id=self.grp['id'],
                                                     name=group_snapshot_name)
        with self.rbac_utils.override_role(self):
            self.group_snapshots_client.reset_group_snapshot_status(
                group_snapshot['id'], 'error')

        waiters.wait_for_volume_resource_status(
            self.group_snapshots_client, group_snapshot['id'], 'error')



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.identity import rbac_base


class IdentityRolesV3RbacTest(rbac_base.BaseIdentityV3RbacTest):

    @classmethod
    def resource_setup(cls):
        super(IdentityRolesV3RbacTest, cls).resource_setup()
        cls.domain = cls.setup_test_domain()
        cls.project = cls.setup_test_project()
        cls.group = cls.setup_test_group()
        cls.role = cls.setup_test_role()
        cls.implies_role = cls.setup_test_role()
        cls.user = cls.setup_test_user()

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:create_role")
    @decorators.idempotent_id('0f148510-63bf-11e6-1395-080044d0d904')
    def test_create_role(self):
        with self.rbac_utils.override_role(self):
            self.setup_test_role()

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:update_role")
    @decorators.idempotent_id('0f148510-63bf-11e6-1395-080044d0d905')
    def test_update_role(self):
        new_role_name = data_utils.rand_name(
            self.__class__.__name__ + '-test_update_role')

        with self.rbac_utils.override_role(self):
            self.roles_client.update_role(self.role['id'],
                                          name=new_role_name)

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:delete_role")
    @decorators.idempotent_id('0f148510-63bf-11e6-1395-080044d0d906')
    def test_delete_role(self):
        role = self.setup_test_role()

        with self.rbac_utils.override_role(self):
            self.roles_client.delete_role(role['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:get_role")
    @decorators.idempotent_id('0f148510-63bf-11e6-1395-080044d0d907')
    def test_show_role(self):
        with self.rbac_utils.override_role(self):
            self.roles_client.show_role(self.role['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:list_roles")
    @decorators.idempotent_id('0f148510-63bf-11e6-1395-080044d0d908')
    def test_list_roles(self):
        with self.rbac_utils.override_role(self):
            self.roles_client.list_roles()

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:create_grant")
    @decorators.idempotent_id('0f148510-63bf-11e6-1395-080044d0d909')
    def test_create_user_role_on_project(self):
        with self.rbac_utils.override_role(self):
            self.roles_client.create_user_role_on_project(
                self.project['id'],
                self.user['id'],
                self.role['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.roles_client.delete_role_from_user_on_project,
                        self.project['id'],
                        self.user['id'],
                        self.role['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:create_grant")
    @decorators.idempotent_id('0f148510-63bf-11e6-1395-080044d0d90c')
    def test_create_group_role_on_project(self):
        with self.rbac_utils.override_role(self):
            self.roles_client.create_group_role_on_project(
                self.project['id'],
                self.group['id'],
                self.role['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.roles_client.delete_role_from_group_on_project,
                        self.project['id'],
                        self.group['id'],
                        self.role['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:create_grant")
    @decorators.idempotent_id('0f148510-63bf-11e6-1395-080044d0d90f')
    def test_create_user_role_on_domain(self):
        with self.rbac_utils.override_role(self):
            self.roles_client.create_user_role_on_domain(
                self.domain['id'],
                self.user['id'],
                self.role['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.roles_client.delete_role_from_user_on_domain,
                        self.domain['id'],
                        self.user['id'],
                        self.role['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:create_grant")
    @decorators.idempotent_id('0f148510-63bf-11e6-1395-080044d0d912')
    def test_create_group_role_on_domain(self):
        with self.rbac_utils.override_role(self):
            self.roles_client.create_group_role_on_domain(
                self.domain['id'],
                self.group['id'],
                self.role['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.roles_client.delete_role_from_group_on_domain,
                        self.domain['id'],
                        self.group['id'],
                        self.role['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:check_grant")
    @decorators.idempotent_id('22921b1e-1a33-4026-bff9-f236d6dd149c')
    def test_check_user_role_existence_on_project(self):
        self.roles_client.create_user_role_on_project(
            self.project['id'],
            self.user['id'],
            self.role['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.roles_client.delete_role_from_user_on_project,
                        self.project['id'],
                        self.user['id'],
                        self.role['id'])

        with self.rbac_utils.override_role(self):
            self.roles_client.check_user_role_existence_on_project(
                self.project['id'],
                self.user['id'],
                self.role['id'])

    @decorators.idempotent_id('92f8e67d-85bf-407d-9814-edd5664abc47')
    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:check_grant")
    def test_check_user_role_existence_on_domain(self):
        self.roles_client.create_user_role_on_domain(
            self.domain['id'],
            self.user['id'],
            self.role['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.roles_client.delete_role_from_user_on_domain,
                        self.domain['id'],
                        self.user['id'],
                        self.role['id'])

        with self.rbac_utils.override_role(self):
            self.roles_client.check_user_role_existence_on_domain(
                self.domain['id'],
                self.user['id'],
                self.role['id'])

    @decorators.idempotent_id('8738d3d2-8c84-4423-b36c-7c59eaa08b73')
    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:check_grant")
    def test_check_role_from_group_on_project_existence(self):
        self.roles_client.create_group_role_on_project(
            self.project['id'],
            self.group['id'],
            self.role['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.roles_client.delete_role_from_group_on_project,
                        self.project['id'],
                        self.group['id'],
                        self.role['id'])

        with self.rbac_utils.override_role(self):
            self.roles_client.check_role_from_group_on_project_existence(
                self.project['id'],
                self.group['id'],
                self.role['id'])

    @decorators.idempotent_id('e7d73bd0-cf5e-4c0c-9c93-cf53e23232d6')
    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:check_grant")
    def test_check_role_from_group_on_domain_existence(self):
        self.roles_client.create_group_role_on_domain(
            self.domain['id'],
            self.group['id'],
            self.role['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.roles_client.delete_role_from_group_on_domain,
                        self.domain['id'],
                        self.group['id'],
                        self.role['id'])

        with self.rbac_utils.override_role(self):
            self.roles_client.check_role_from_group_on_domain_existence(
                self.domain['id'],
                self.group['id'],
                self.role['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:revoke_grant")
    @decorators.idempotent_id('0f148510-63bf-11e6-1395-080044d0d90a')
    def test_delete_role_from_user_on_project(self):
        self.roles_client.create_user_role_on_project(
            self.project['id'],
            self.user['id'],
            self.role['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.roles_client.delete_role_from_user_on_project,
                        self.project['id'],
                        self.user['id'],
                        self.role['id'])

        with self.rbac_utils.override_role(self):
            self.roles_client.delete_role_from_user_on_project(
                self.project['id'],
                self.user['id'],
                self.role['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:revoke_grant")
    @decorators.idempotent_id('0f148510-63bf-11e6-1395-080044d0d90d')
    def test_delete_role_from_group_on_project(self):
        self.roles_client.create_group_role_on_project(
            self.project['id'],
            self.group['id'],
            self.role['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.roles_client.delete_role_from_group_on_project,
                        self.project['id'],
                        self.group['id'],
                        self.role['id'])

        with self.rbac_utils.override_role(self):
            self.roles_client.delete_role_from_group_on_project(
                self.project['id'],
                self.group['id'],
                self.role['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:revoke_grant")
    @decorators.idempotent_id('0f148510-63bf-11e6-1395-080044d0d910')
    def test_delete_role_from_user_on_domain(self):
        self.roles_client.create_user_role_on_domain(
            self.domain['id'],
            self.user['id'],
            self.role['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.roles_client.delete_role_from_user_on_domain,
                        self.domain['id'],
                        self.user['id'],
                        self.role['id'])

        with self.rbac_utils.override_role(self):
            self.roles_client.delete_role_from_user_on_domain(
                self.domain['id'],
                self.user['id'],
                self.role['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:revoke_grant")
    @decorators.idempotent_id('0f148510-63bf-11e6-1395-080044d0d913')
    def test_delete_role_from_group_on_domain(self):
        self.roles_client.create_group_role_on_domain(
            self.domain['id'],
            self.group['id'],
            self.role['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.roles_client.delete_role_from_group_on_domain,
                        self.domain['id'],
                        self.group['id'],
                        self.role['id'])

        with self.rbac_utils.override_role(self):
            self.roles_client.delete_role_from_group_on_domain(
                self.domain['id'],
                self.group['id'],
                self.role['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:list_grants")
    @decorators.idempotent_id('0f148510-63bf-11e6-1395-080044d0d90b')
    def test_list_user_roles_on_project(self):
        with self.rbac_utils.override_role(self):
            self.roles_client.list_user_roles_on_project(
                self.project['id'],
                self.user['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:list_grants")
    @decorators.idempotent_id('0f148510-63bf-11e6-1395-080044d0d90e')
    def test_list_group_roles_on_project(self):
        with self.rbac_utils.override_role(self):
            self.roles_client.list_group_roles_on_project(
                self.project['id'],
                self.group['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:list_grants")
    @decorators.idempotent_id('0f148510-63bf-11e6-1395-080044d0d911')
    def test_list_user_roles_on_domain(self):
        with self.rbac_utils.override_role(self):
            self.roles_client.list_user_roles_on_domain(
                self.domain['id'],
                self.user['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:list_grants")
    @decorators.idempotent_id('0f148510-63bf-11e6-1395-080044d0d914')
    def test_list_group_roles_on_domain(self):
        with self.rbac_utils.override_role(self):
            self.roles_client.list_group_roles_on_domain(
                self.domain['id'],
                self.group['id'])

    @decorators.idempotent_id('2aef3eaa-8156-4962-a01d-c9bb0e499e15')
    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:create_implied_role")
    def test_create_role_inference_rule(self):
        with self.rbac_utils.override_role(self):
            self.roles_client.create_role_inference_rule(
                self.role['id'], self.implies_role['id'])
        self.addCleanup(self.roles_client.delete_role_inference_rule,
                        self.role['id'], self.implies_role['id'])

    @decorators.idempotent_id('83f997b2-55c4-4894-b1f2-e175b19d1fa5')
    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:get_implied_role")
    def test_show_role_inference_rule(self):
        self.roles_client.create_role_inference_rule(
            self.role['id'], self.implies_role['id'])
        self.addCleanup(self.roles_client.delete_role_inference_rule,
                        self.role['id'], self.implies_role['id'])

        with self.rbac_utils.override_role(self):
            self.roles_client.show_role_inference_rule(
                self.role['id'], self.implies_role['id'])

    @decorators.idempotent_id('f7bb39bf-0b06-468e-a8b0-60a4fb1f258d')
    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:list_implied_roles")
    def test_list_role_inferences_rules(self):
        with self.rbac_utils.override_role(self):
            self.roles_client.list_role_inferences_rules(self.role['id'])

    @decorators.idempotent_id('eca2d502-09bb-45cd-9773-bce2e7bcddd1')
    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:check_implied_role")
    def test_check_role_inference_rule(self):
        self.roles_client.create_role_inference_rule(
            self.role['id'], self.implies_role['id'])
        self.addCleanup(self.roles_client.delete_role_inference_rule,
                        self.role['id'], self.implies_role['id'])

        with self.rbac_utils.override_role(self):
            self.roles_client.check_role_inference_rule(
                self.role['id'], self.implies_role['id'])

    @decorators.idempotent_id('13a5db1e-dd4a-4ca1-81ec-d5452aaaf54b')
    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:delete_implied_role")
    def test_delete_role_inference_rule(self):
        self.roles_client.create_role_inference_rule(
            self.role['id'], self.implies_role['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.roles_client.delete_role_inference_rule,
                        self.role['id'], self.implies_role['id'])

        with self.rbac_utils.override_role(self):
            self.roles_client.delete_role_inference_rule(
                self.role['id'], self.implies_role['id'])

    @decorators.idempotent_id('05869f2b-4dd4-425a-905e-eec9a6f06374')
    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:list_role_inference_rules")
    def test_list_all_role_inference_rules(self):
        with self.rbac_utils.override_role(self):
            self.roles_client.list_all_role_inference_rules()



# Copyright 2013 IBM Corp.
# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import os
import re

import pep8


PYTHON_CLIENTS = ['cinder', 'glance', 'keystone', 'nova', 'swift', 'neutron',
                  'ironic', 'heat', 'sahara']

PYTHON_CLIENT_RE = re.compile('import (%s)client' % '|'.join(PYTHON_CLIENTS))
TEST_DEFINITION = re.compile(r'^\s*def test.*')
SETUP_TEARDOWN_CLASS_DEFINITION = re.compile(r'^\s+def (setUp|tearDown)Class')
SCENARIO_DECORATOR = re.compile(r'\s*@.*services\((.*)\)')
VI_HEADER_RE = re.compile(r"^#\s+vim?:.+")
RAND_NAME_HYPHEN_RE = re.compile(r".*rand_name\(.+[\-\_][\"\']\)")
MUTABLE_DEFAULT_ARGS = re.compile(r"^\s*def .+\((.+=\{\}|.+=\[\])")
TESTTOOLS_SKIP_DECORATOR = re.compile(r'\s*@testtools\.skip\((.*)\)')
CLASS = re.compile(r"^class .+")
RBAC_CLASS_NAME_RE = re.compile(r'class .+RbacTest')
RULE_VALIDATION_DECORATOR = re.compile(
    r'\s*@rbac_rule_validation.action\(.*')
IDEMPOTENT_ID_DECORATOR = re.compile(r'\s*@decorators\.idempotent_id\((.*)\)')

have_rbac_decorator = False


def import_no_clients_in_api_tests(physical_line, filename):
    """Check for client imports from patrole_tempest_plugin/tests/api

    T102: Cannot import OpenStack python clients
    """
    if "patrole_tempest_plugin/tests/api" in filename:
        res = PYTHON_CLIENT_RE.match(physical_line)
        if res:
            return (physical_line.find(res.group(1)),
                    ("T102: python clients import not allowed "
                     "in patrole_tempest_plugin/tests/api/* or "
                     "patrole_tempest_plugin/tests/scenario/* tests"))


def no_setup_teardown_class_for_tests(physical_line, filename):
    """Check that tests do not use setUpClass/tearDownClass

    T105: Tests cannot use setUpClass/tearDownClass
    """
    if pep8.noqa(physical_line):
        return

    if SETUP_TEARDOWN_CLASS_DEFINITION.match(physical_line):
        return (physical_line.find('def'),
                "T105: (setUp|tearDown)Class can not be used in tests")


def no_vi_headers(physical_line, line_number, lines):
    """Check for vi editor configuration in source files.

    By default vi modelines can only appear in the first or
    last 5 lines of a source file.

    T106
    """
    # NOTE(gilliard): line_number is 1-indexed
    if line_number <= 5 or line_number > len(lines) - 5:
        if VI_HEADER_RE.match(physical_line):
            return 0, "T106: Don't put vi configuration in source files"


def service_tags_not_in_module_path(physical_line, filename):
    """Check that a service tag isn't in the module path

    A service tag should only be added if the service name isn't already in
    the module path.

    T107
    """
    matches = SCENARIO_DECORATOR.match(physical_line)
    if matches:
        services = matches.group(1).split(',')
        for service in services:
            service_name = service.strip().strip("'")
            modulepath = os.path.split(filename)[0]
            if service_name in modulepath:
                return (physical_line.find(service_name),
                        "T107: service tag should not be in path")


def no_hyphen_at_end_of_rand_name(logical_line, filename):
    """Check no hyphen at the end of rand_name() argument

    T108
    """
    msg = "T108: hyphen should not be specified at the end of rand_name()"
    if RAND_NAME_HYPHEN_RE.match(logical_line):
        return 0, msg


def no_mutable_default_args(logical_line):
    """Check that mutable object isn't used as default argument

    N322: Method's default argument shouldn't be mutable
    """
    msg = "N322: Method's default argument shouldn't be mutable!"
    if MUTABLE_DEFAULT_ARGS.match(logical_line):
        yield (0, msg)


def no_testtools_skip_decorator(logical_line):
    """Check that methods do not have the testtools.skip decorator

    T109
    """
    if TESTTOOLS_SKIP_DECORATOR.match(logical_line):
        yield (0, "T109: Cannot use testtools.skip decorator; instead use "
               "decorators.skip_because from tempest.lib")


def use_rand_uuid_instead_of_uuid4(logical_line, filename):
    """Check that tests use data_utils.rand_uuid() instead of uuid.uuid4()

    T113
    """
    if 'uuid.uuid4()' not in logical_line:
        return

    msg = ("T113: Tests should use data_utils.rand_uuid()/rand_uuid_hex() "
           "instead of uuid.uuid4()/uuid.uuid4().hex")
    yield (0, msg)


def no_rbac_rule_validation_decorator(physical_line, filename):
    """Check that each test has the ``rbac_rule_validation.action`` decorator.

    Checks whether the test function has "@rbac_rule_validation.action"
    above it; otherwise checks that it has "@decorators.idempotent_id" above
    it and "@rbac_rule_validation.action" above that.

    Assumes that ``rbac_rule_validation.action`` decorator is either the first
    or second decorator above the test function; otherwise this check fails.

    P100
    """
    global have_rbac_decorator

    if ("patrole_tempest_plugin/tests/api" in filename or
            "patrole_tempest_plugin/tests/scenario" in filename):

        if RULE_VALIDATION_DECORATOR.match(physical_line):
            have_rbac_decorator = True
            return

        if TEST_DEFINITION.match(physical_line):
            if not have_rbac_decorator:
                return (0, "Must use rbac_rule_validation.action "
                           "decorator for API and scenario tests")

            have_rbac_decorator = False


def no_rbac_suffix_in_test_filename(filename):
    """Check that RBAC filenames end with "_rbac" suffix.

    P101
    """
    if "patrole_tempest_plugin/tests/api" in filename:

        if filename.endswith('rbac_base.py'):
            return

        if not filename.endswith('_rbac.py'):
            return 0, "RBAC test filenames must end in _rbac suffix"


def no_rbac_test_suffix_in_test_class_name(physical_line, filename):
    """Check that RBAC class names end with "RbacTest"

    P102
    """
    if "patrole_tempest_plugin/tests/api" in filename:

        if filename.endswith('rbac_base.py'):
            return

        if CLASS.match(physical_line):
            if not RBAC_CLASS_NAME_RE.match(physical_line):
                return 0, "RBAC test class names must end in 'RbacTest'"


def no_client_alias_in_test_cases(logical_line, filename):
    """Check that test cases don't use "self.client" to define a client.

    P103
    """
    if "patrole_tempest_plugin/tests/api" in filename:
        if "self.client" in logical_line or "cls.client" in logical_line:
            return 0, "Do not use 'self.client' as a service client alias"


def factory(register):
    register(import_no_clients_in_api_tests)
    register(no_setup_teardown_class_for_tests)
    register(no_vi_headers)
    register(no_hyphen_at_end_of_rand_name)
    register(no_mutable_default_args)
    register(no_testtools_skip_decorator)
    register(use_rand_uuid_instead_of_uuid4)
    register(service_tags_not_in_module_path)
    register(no_rbac_rule_validation_decorator)
    register(no_rbac_suffix_in_test_filename)
    register(no_rbac_test_suffix_in_test_class_name)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
import sys
import testtools

from oslo_utils import excutils
import six

from tempest import config
from tempest.lib import exceptions
from tempest import test

from patrole_tempest_plugin import policy_authority
from patrole_tempest_plugin import rbac_exceptions
from patrole_tempest_plugin import rbac_utils
from patrole_tempest_plugin import requirements_authority

CONF = config.CONF
LOG = logging.getLogger(__name__)

_SUPPORTED_ERROR_CODES = [403, 404]

RBACLOG = logging.getLogger('rbac_reporting')


def action(service, rule='', admin_only=False, expected_error_code=403,
           extra_target_data=None):
    """A decorator for verifying OpenStack policy enforcement.

    A decorator which allows for positive and negative RBAC testing. Given:

    * an OpenStack service,
    * a policy action (``rule``) enforced by that service, and
    * the test role defined by ``[patrole] rbac_test_role``

    determines whether the test role has sufficient permissions to perform an
    API call that enforces the ``rule``.

    This decorator should only be applied to an instance or subclass of
    ``tempest.test.BaseTestCase``.

    The result from ``_is_authorized`` is used to determine the *expected*
    test result. The *actual* test result is determined by running the
    Tempest test this decorator applies to.

    Below are the following possibilities from comparing the *expected* and
    *actual* results:

    1) If *expected* is True and the test passes (*actual*), this is a success.
    2) If *expected* is True and the test fails (*actual*), this results in a
       `Forbidden` exception failure.
    3) If *expected* is False and the test passes (*actual*), this results in
       an `OverPermission` exception failure.
    4) If *expected* is False and the test fails (*actual*), this is a success.

    As such, negative and positive testing can be applied using this decorator.

    :param service: An OpenStack service. Examples: "nova" or "neutron".
    :param rule: A policy action defined in a policy.json file (or in
        code).

        .. note::

            Patrole currently only supports custom JSON policy files.

    :param admin_only: Skips over ``oslo.policy`` check because the policy
        action defined by ``rule`` is not enforced by the service's policy
        enforcement engine. For example, Keystone v2 performs an admin check
        for most of its endpoints. If True, ``rule`` is effectively ignored.
    :param expected_error_code: Overrides default value of 403 (Forbidden)
        with endpoint-specific error code. Currently only supports 403 and 404.
        Support for 404 is needed because some services, like Neutron,
        intentionally throw a 404 for security reasons.

        .. warning::

            A 404 should not be provided *unless* the endpoint masks a
            ``Forbidden`` exception as a ``NotFound`` exception.

    :param extra_target_data: Dictionary, keyed with ``oslo.policy`` generic
        check names, whose values are string literals that reference nested
        ``tempest.test.BaseTestCase`` attributes. Used by ``oslo.policy`` for
        performing matching against attributes that are sent along with the API
        calls. Example::

            extra_target_data={
                "target.token.user_id":
                "os_alt.auth_provider.credentials.user_id"
            })

    :raises NotFound: If ``service`` is invalid.
    :raises Forbidden: For item (2) above.
    :raises RbacOverPermission: For item (3) above.

    Examples::

        @rbac_rule_validation.action(
            service="nova", rule="os_compute_api:os-agents")
        def test_list_agents_rbac(self):
            # The call to `switch_role` is mandatory.
            self.rbac_utils.switch_role(self, toggle_rbac_role=True)
            self.agents_client.list_agents()
    """

    if extra_target_data is None:
        extra_target_data = {}

    def decorator(test_func):
        role = CONF.patrole.rbac_test_role

        def wrapper(*args, **kwargs):
            if args and isinstance(args[0], test.BaseTestCase):
                test_obj = args[0]
            else:
                raise rbac_exceptions.RbacResourceSetupFailed(
                    '`rbac_rule_validation` decorator can only be applied to '
                    'an instance of `tempest.test.BaseTestCase`.')

            allowed = _is_authorized(test_obj, service, rule,
                                     extra_target_data, admin_only)

            expected_exception, irregular_msg = _get_exception_type(
                expected_error_code)

            test_status = 'Allowed'

            try:
                test_func(*args, **kwargs)
            except rbac_exceptions.RbacInvalidService as e:
                msg = ("%s is not a valid service." % service)
                test_status = ('Error, %s' % (msg))
                LOG.error(msg)
                raise exceptions.NotFound(
                    "%s RbacInvalidService was: %s" % (msg, e))
            except (expected_exception,
                    rbac_exceptions.RbacConflictingPolicies,
                    rbac_exceptions.RbacMalformedResponse) as e:
                test_status = 'Denied'
                if irregular_msg:
                    LOG.warning(irregular_msg.format(rule, service))
                if allowed:
                    msg = ("Role %s was not allowed to perform %s." %
                           (role, rule))
                    LOG.error(msg)
                    raise exceptions.Forbidden(
                        "%s Exception was: %s" % (msg, e))
            except Exception as e:
                with excutils.save_and_reraise_exception():
                    exc_info = sys.exc_info()
                    error_details = six.text_type(exc_info[1])
                    msg = ("An unexpected exception has occurred during test: "
                           "%s. Exception was: %s" % (test_func.__name__,
                                                      error_details))
                    test_status = 'Error, %s' % (error_details)
                    LOG.error(msg)
            else:
                if not allowed:
                    LOG.error("Role %s was allowed to perform %s",
                              role, rule)
                    raise rbac_exceptions.RbacOverPermission(
                        "OverPermission: Role %s was allowed to perform %s" %
                        (role, rule))
            finally:
                if CONF.patrole_log.enable_reporting:
                    RBACLOG.info(
                        "[Service]: %s, [Test]: %s, [Rule]: %s, "
                        "[Expected]: %s, [Actual]: %s",
                        service, test_func.__name__, rule,
                        "Allowed" if allowed else "Denied",
                        test_status)

        _wrapper = testtools.testcase.attr(role)(wrapper)
        return _wrapper
    return decorator


def _is_authorized(test_obj, service, rule, extra_target_data, admin_only):
    """Validates whether current RBAC role has permission to do policy action.

    :param test_obj: An instance or subclass of ``tempest.test.BaseTestCase``.
    :param service: The OpenStack service that enforces ``rule``.
    :param rule: The name of the policy action. Examples include
        "identity:create_user" or "os_compute_api:os-agents".
    :param extra_target_data: Dictionary, keyed with ``oslo.policy`` generic
        check names, whose values are string literals that reference nested
        ``tempest.test.BaseTestCase`` attributes. Used by ``oslo.policy`` for
        performing matching against attributes that are sent along with the API
        calls.
    :param admin_only: Skips over ``oslo.policy`` check because the policy
        action defined by ``rule`` is not enforced by the service's policy
        enforcement engine. For example, Keystone v2 performs an admin check
        for most of its endpoints. If True, ``rule`` is effectively ignored.

    :returns: True if the current RBAC role can perform the policy action,
        else False.

    :raises RbacResourceSetupFailed: If `project_id` or `user_id` are missing
        from the `auth_provider` attribute in `test_obj`.
    :raises RbacParsingException: if ``[patrole] strict_policy_check`` is True
        and the ``rule`` does not exist in the system.
    :raises skipException: If ``[patrole] strict_policy_check`` is False and
        the ``rule`` does not exist in the system.
    """

    if admin_only:
        LOG.info("As admin_only is True, only admin role should be "
                 "allowed to perform the API. Skipping oslo.policy "
                 "check for policy action {0}.".format(rule))
        return rbac_utils.is_admin()

    try:
        project_id = test_obj.os_primary.credentials.project_id
        user_id = test_obj.os_primary.credentials.user_id
    except AttributeError as e:
        msg = ("{0}: project_id or user_id not found in os_primary.credentials"
               .format(e))
        LOG.error(msg)
        raise rbac_exceptions.RbacResourceSetupFailed(msg)

    try:
        role = CONF.patrole.rbac_test_role
        # Test RBAC against custom requirements. Otherwise use oslo.policy.
        if CONF.patrole.test_custom_requirements:
            authority = requirements_authority.RequirementsAuthority(
                CONF.patrole.custom_requirements_file, service)
        else:
            formatted_target_data = _format_extra_target_data(
                test_obj, extra_target_data)
            authority = policy_authority.PolicyAuthority(
                project_id, user_id, service,
                extra_target_data=formatted_target_data)
        is_allowed = authority.allowed(rule, role)

        if is_allowed:
            LOG.debug("[Action]: %s, [Role]: %s is allowed!", rule,
                      role)
        else:
            LOG.debug("[Action]: %s, [Role]: %s is NOT allowed!",
                      rule, role)
        return is_allowed
    except rbac_exceptions.RbacParsingException as e:
        if CONF.patrole.strict_policy_check:
            raise e
        else:
            raise testtools.TestCase.skipException(str(e))
    return False


def _get_exception_type(expected_error_code=403):
    """Dynamically calculate the expected exception to be caught.

    Dynamically calculate the expected exception to be caught by the test case.
    Only ``Forbidden`` and ``NotFound`` exceptions are permitted. ``NotFound``
    is supported because Neutron, for security reasons, masks ``Forbidden``
    exceptions as ``NotFound`` exceptions.

    :param expected_error_code: the integer representation of the expected
        exception to be caught. Must be contained in
        ``_SUPPORTED_ERROR_CODES``.
    :returns: tuple of the exception type corresponding to
        ``expected_error_code`` and a message explaining that a non-Forbidden
        exception was expected, if applicable.
    """
    expected_exception = None
    irregular_msg = None

    if not isinstance(expected_error_code, six.integer_types) \
            or expected_error_code not in _SUPPORTED_ERROR_CODES:
        msg = ("Please pass an expected error code. Currently "
               "supported codes: {0}".format(_SUPPORTED_ERROR_CODES))
        LOG.error(msg)
        raise rbac_exceptions.RbacInvalidErrorCode(msg)

    if expected_error_code == 403:
        expected_exception = exceptions.Forbidden
    elif expected_error_code == 404:
        expected_exception = exceptions.NotFound
        irregular_msg = ("NotFound exception was caught for policy action "
                         "{0}. The service {1} throws a 404 instead of a 403, "
                         "which is irregular.")

    return expected_exception, irregular_msg


def _format_extra_target_data(test_obj, extra_target_data):
    """Formats the "extra_target_data" dictionary with correct test data.

    Before being formatted, "extra_target_data" is a dictionary that maps a
    policy string like "trust.trustor_user_id" to a nested list of
    ``tempest.test.BaseTestCase`` attributes. For example, the attribute list
    in::

      "trust.trustor_user_id": "os.auth_provider.credentials.user_id"

    is parsed by iteratively calling ``getattr`` until the value of "user_id"
    is resolved. The resulting dictionary returns::

      "trust.trustor_user_id": "the user_id of the `os_primary` credential"

    :param test_obj: An instance or subclass of ``tempest.test.BaseTestCase``.
    :param extra_target_data: Dictionary, keyed with ``oslo.policy`` generic
        check names, whose values are string literals that reference nested
        ``tempest.test.BaseTestCase`` attributes. Used by ``oslo.policy`` for
        performing matching against attributes that are sent along with the API
        calls.
    :returns: Dictionary containing additional object data needed by
        ``oslo.policy`` to validate generic checks.
    """
    attr_value = test_obj
    formatted_target_data = {}

    for user_attribute, attr_string in extra_target_data.items():
        attrs = attr_string.split('.')
        for attr in attrs:
            attr_value = getattr(attr_value, attr)
        formatted_target_data[user_attribute] = attr_value

    return formatted_target_data



#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.image import rbac_base


class ImageResourceTypesRbacTest(rbac_base.BaseV2ImageRbacTest):

    @classmethod
    def resource_setup(cls):
        super(ImageResourceTypesRbacTest, cls).resource_setup()
        cls.namespace_name = data_utils.rand_name(
            cls.__name__ + '-test-ns')
        cls.namespaces_client.create_namespace(
            namespace=cls.namespace_name,
            protected=False)
        cls.addClassResourceCleanup(
            test_utils.call_and_ignore_notfound_exc,
            cls.namespaces_client.delete_namespace, cls.namespace_name)

    @rbac_rule_validation.action(service="glance",
                                 rule="list_metadef_resource_types")
    @decorators.idempotent_id('0416fc4d-cfdc-447b-88b6-d9f1dd0382f7')
    def test_list_metadef_resource_types(self):
        """List Metadef Resource Type Image Test

        RBAC test for the glance list_metadef_resource_type policy.
        """
        with self.rbac_utils.override_role(self):
            self.resource_types_client.list_resource_types()

    @rbac_rule_validation.action(service="glance",
                                 rule="get_metadef_resource_type")
    @decorators.idempotent_id('3698d53c-71ae-4803-a2c3-c272c054f25c')
    def test_get_metadef_resource_type(self):
        """Get Metadef Resource Type Image Test

        RBAC test for the glance get_metadef_resource_type policy.
        """
        with self.rbac_utils.override_role(self):
            self.resource_types_client.list_resource_type_association(
                self.namespace_name)

    @rbac_rule_validation.action(service="glance",
                                 rule="add_metadef_resource_type_association")
    @decorators.idempotent_id('ef9fbc60-3e28-4164-a25c-d30d892f7939')
    def test_add_metadef_resource_type(self):
        type_name = data_utils.rand_name()
        with self.rbac_utils.override_role(self):
            self.resource_types_client.create_resource_type_association(
                self.namespace_name, name=type_name)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Fixtures for Patrole tests."""
from __future__ import absolute_import

import fixtures
import mock
import time

from tempest import clients
from tempest.common import credentials_factory as credentials
from tempest import config
from tempest import test

from patrole_tempest_plugin import rbac_utils


CONF = config.CONF


class ConfPatcher(fixtures.Fixture):
    """Fixture to patch and restore global CONF. Adopted from Nova.

    This also resets overrides for everything that is patched during
    its teardown.
    """

    def __init__(self, **kwargs):
        """Constructor

        :params group: if specified all config options apply to that group.
        :params **kwargs: the rest of the kwargs are processed as a
            set of key/value pairs to be set as configuration override.
        """
        super(ConfPatcher, self).__init__()
        self.group = kwargs.pop('group', None)
        self.args = kwargs

    def setUp(self):
        super(ConfPatcher, self).setUp()
        for k, v in self.args.items():
            self.addCleanup(CONF.clear_override, k, self.group)
            CONF.set_override(k, v, self.group)


class RbacUtilsFixture(fixtures.Fixture):
    """Fixture for `RbacUtils` class."""

    USER_ID = mock.sentinel.user_id
    PROJECT_ID = mock.sentinel.project_id

    def setUp(self):
        super(RbacUtilsFixture, self).setUp()

        self.useFixture(ConfPatcher(rbac_test_role='member', group='patrole'))
        self.useFixture(ConfPatcher(
            admin_role='admin', auth_version='v3', group='identity'))

        test_obj_kwargs = {
            'os_primary.credentials.user_id': self.USER_ID,
            'os_primary.credentials.tenant_id': self.PROJECT_ID,
            'os_primary.credentials.project_id': self.PROJECT_ID,
            'get_identity_version.return_value': 'v3'
        }
        self.mock_test_obj = mock.Mock(
            __name__='patrole_unit_test', spec=test.BaseTestCase,
            os_primary=mock.Mock(), **test_obj_kwargs)

        # Mock out functionality that can't be used by unit tests. Mocking out
        # time.sleep is a test optimization.
        self.mock_time = mock.patch.object(
            rbac_utils, 'time', __name__='mock_time', spec=time).start()
        mock.patch.object(credentials, 'get_configured_admin_credentials',
                          spec=object).start()
        mock_admin_mgr = mock.patch.object(
            clients, 'Manager', spec=clients.Manager,
            roles_v3_client=mock.Mock(), roles_client=mock.Mock()).start()
        self.roles_v3_client = mock_admin_mgr.return_value.roles_v3_client

        self.set_roles(['admin', 'member'], [])

    def override_role(self, *role_toggles):
        """Instantiate `rbac_utils.RbacUtils` and call `override_role`.

        Create an instance of `rbac_utils.RbacUtils` and call `override_role`
        for each boolean value in `role_toggles`. The number of calls to
        `override_role` is always 1 + len(`role_toggles`) because the
        `rbac_utils.RbacUtils` constructor automatically calls `override_role`.

        :param role_toggles: the list of boolean values iterated over and
            passed to `override_role`.
        """
        _rbac_utils = rbac_utils.RbacUtils(self.mock_test_obj)

        for role_toggle in role_toggles:
            _rbac_utils._override_role(self.mock_test_obj, role_toggle)
            # NOTE(felipemonteiro): Simulate that a role switch has occurred
            # by updating the user's current role to the new role. This means
            # that all API actions involved during a role switch -- listing,
            # deleting and adding roles -- are executed, making it easier to
            # assert that mock calls were called as expected.
            new_role = 'member' if role_toggle else 'admin'
            self.set_roles(['admin', 'member'], [new_role])

    def set_roles(self, roles, roles_on_project=None):
        """Set the list of available roles in the system.

        :param roles: List of roles returned by ``list_roles``.
        :param roles_on_project: List of roles returned by
            ``list_user_roles_on_project``.
        :returns: None.
        """
        if not roles_on_project:
            roles_on_project = []
        if not isinstance(roles, list):
            roles = [roles]
        if not isinstance(roles_on_project, list):
            roles_on_project = [roles_on_project]

        available_roles = {
            'roles': [{'name': role, 'id': '%s_id' % role} for role in roles]
        }
        available_project_roles = {
            'roles': [{'name': role, 'id': '%s_id' % role}
                      for role in roles_on_project]
        }

        self.roles_v3_client.list_roles.return_value = available_roles
        self.roles_v3_client.list_user_roles_on_project.return_value = (
            available_project_roles)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.identity import rbac_base


class EndpointFilterGroupsV3RbacTest(rbac_base.BaseIdentityV3RbacTest):

    interface = 'public'

    @classmethod
    def resource_setup(cls):
        super(EndpointFilterGroupsV3RbacTest, cls).resource_setup()
        cls.service_id = cls.setup_test_service()['id']

    def setUp(self):
        super(EndpointFilterGroupsV3RbacTest, self).setUp()
        self.endpoint_group_id = self._create_endpoint_group()

    def _create_endpoint_group(self, ignore_not_found=False):
        # Create an endpoint group
        ep_group_name = data_utils.rand_name(
            self.__class__.__name__ + '-EPFilterGroup')
        filters = {
            'filters': {
                'interface': self.interface,
                'service_id': self.service_id
            }
        }
        endpoint_group = self.endpoint_groups_client.create_endpoint_group(
            name=ep_group_name, **filters)['endpoint_group']

        if ignore_not_found:
            self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                            self.endpoint_groups_client.delete_endpoint_group,
                            endpoint_group['id'])
        else:
            self.addCleanup(self.endpoint_groups_client.delete_endpoint_group,
                            endpoint_group['id'])

        return endpoint_group['id']

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:create_endpoint_group")
    @decorators.idempotent_id('b4765906-52ec-477b-b441-a8508ced68e3')
    def test_create_endpoint_group(self):
        with self.rbac_utils.override_role(self):
            self._create_endpoint_group(ignore_not_found=True)

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:list_endpoint_groups")
    @decorators.idempotent_id('089aa3a7-ba1f-4f70-a1cf-f298a845058a')
    def test_list_endpoint_groups(self):
        with self.rbac_utils.override_role(self):
            self.endpoint_groups_client.list_endpoint_groups()

    @decorators.idempotent_id('5c16368d-1485-4c28-9803-db3fa3510623')
    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:get_endpoint_group")
    def test_check_endpoint_group(self):
        with self.rbac_utils.override_role(self):
            self.endpoint_groups_client.check_endpoint_group(
                self.endpoint_group_id)

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:get_endpoint_group")
    @decorators.idempotent_id('bd2b6fb8-661f-4255-84b2-50fea4a1dc61')
    def test_show_endpoint_group(self):
        with self.rbac_utils.override_role(self):
            self.endpoint_groups_client.show_endpoint_group(
                self.endpoint_group_id)

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:update_endpoint_group")
    @decorators.idempotent_id('028b9198-ec35-4bd5-8f72-e23dfb7a0c8e')
    def test_update_endpoint_group(self):
        updated_name = data_utils.rand_name(
            self.__class__.__name__ + '-EPFilterGroup')

        with self.rbac_utils.override_role(self):
            self.endpoint_groups_client.update_endpoint_group(
                self.endpoint_group_id, name=updated_name)

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:delete_endpoint_group")
    @decorators.idempotent_id('88cc105e-70d9-48ac-927e-200ef41e070c')
    def test_delete_endpoint_group(self):
        endpoint_group_id = self._create_endpoint_group(ignore_not_found=True)

        with self.rbac_utils.override_role(self):
            self.endpoint_groups_client.delete_endpoint_group(
                endpoint_group_id)



# Copyright 2017 AT&T Corporation.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

from tempest.lib import exceptions
from tempest.tests import base

from patrole_tempest_plugin import requirements_authority as req_auth


class RequirementsAuthorityTest(base.TestCase):
    def setUp(self):
        super(RequirementsAuthorityTest, self).setUp()
        self.rbac_auth = req_auth.RequirementsAuthority()
        self.current_directory = os.path.dirname(os.path.realpath(__file__))
        self.yaml_test_file = os.path.join(self.current_directory,
                                           'resources',
                                           'rbac_roles.yaml')
        self.expected_result = {'test:create': ['test_member', '_member_'],
                                'test:create2': ['test_member']}

    def test_requirements_auth_init(self):
        rbac_auth = req_auth.RequirementsAuthority(self.yaml_test_file, 'Test')
        self.assertEqual(self.expected_result, rbac_auth.roles_dict)

    def test_auth_allowed_empty_roles(self):
        self.rbac_auth.roles_dict = None
        self.assertRaises(exceptions.InvalidConfiguration,
                          self.rbac_auth.allowed, "", "")

    def test_auth_allowed_role_in_api(self):
        self.rbac_auth.roles_dict = {'api': ['_member_']}
        self.assertTrue(self.rbac_auth.allowed("api", "_member_"))

    def test_auth_allowed_role_not_in_api(self):
        self.rbac_auth.roles_dict = {'api': ['_member_']}
        self.assertFalse(self.rbac_auth.allowed("api", "support_member"))

    def test_parser_get_allowed_except_keyerror(self):
        self.rbac_auth.roles_dict = {}
        self.assertRaises(KeyError, self.rbac_auth.allowed,
                          "api", "support_member")

    def test_parser_init(self):
        req_auth.RequirementsParser(self.yaml_test_file)
        self.assertEqual([{'Test': self.expected_result}],
                         req_auth.RequirementsParser.Inner._rbac_map)

    def test_parser_role_in_api(self):
        req_auth.RequirementsParser.Inner._rbac_map = \
            [{'Test': self.expected_result}]
        self.rbac_auth.roles_dict = req_auth.RequirementsParser.parse("Test")

        self.assertEqual(self.expected_result, self.rbac_auth.roles_dict)
        self.assertTrue(self.rbac_auth.allowed("test:create2", "test_member"))

    def test_parser_role_not_in_api(self):
        req_auth.RequirementsParser.Inner._rbac_map = \
            [{'Test': self.expected_result}]
        self.rbac_auth.roles_dict = req_auth.RequirementsParser.parse("Test")

        self.assertEqual(self.expected_result, self.rbac_auth.roles_dict)
        self.assertFalse(self.rbac_auth.allowed("test:create2", "_member_"))

    def test_parser_except_invalid_configuration(self):
        req_auth.RequirementsParser.Inner._rbac_map = \
            [{'Test': self.expected_result}]
        self.rbac_auth.roles_dict = \
            req_auth.RequirementsParser.parse("Failure")

        self.assertIsNone(self.rbac_auth.roles_dict)
        self.assertRaises(exceptions.InvalidConfiguration,
                          self.rbac_auth.allowed, "", "")



#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.image import rbac_base


class ImageNamespacesRbacTest(rbac_base.BaseV2ImageRbacTest):

    @rbac_rule_validation.action(service="glance",
                                 rule="add_metadef_namespace")
    @decorators.idempotent_id('e0730ead-b824-4ffc-b774-9469df0e4da6')
    def test_create_metadef_namespace(self):
        """Create Image Metadef Namespace Test

        RBAC test for the glance add_metadef_namespace policy
        """
        namespace_name = data_utils.rand_name(
            self.__class__.__name__ + '-test-ns')
        with self.rbac_utils.override_role(self):
            self.namespaces_client.create_namespace(
                namespace=namespace_name,
                protected=False)
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.namespaces_client.delete_namespace,
            namespace_name)

    @rbac_rule_validation.action(service="glance",
                                 rule="get_metadef_namespaces")
    @decorators.idempotent_id('f0b12538-9047-489e-98a5-2d78f48ce789')
    def test_list_metadef_namespaces(self):
        """List Image Metadef Namespace Test

        RBAC test for the glance get_metadef_namespaces policy
        """
        with self.rbac_utils.override_role(self):
            self.namespaces_client.list_namespaces()

    @rbac_rule_validation.action(service="glance",
                                 rule="modify_metadef_namespace")
    @decorators.idempotent_id('72c14a7e-927d-4f1a-9e1f-25475552922b')
    def test_modify_metadef_namespace(self):
        """Modify Image Metadef Namespace Test

        RBAC test for the glance modify_metadef_namespace policy
        """
        namespace_name = data_utils.rand_name(
            self.__class__.__name__ + '-test-ns')
        body = self.namespaces_client.create_namespace(
            namespace=namespace_name,
            protected=False)
        with self.rbac_utils.override_role(self):
            self.namespaces_client.update_namespace(
                body['namespace'], description="My new description")
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.namespaces_client.delete_namespace,
            namespace_name)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import yaml

from oslo_log import log as logging

from tempest.lib import exceptions

from patrole_tempest_plugin.rbac_utils import RbacAuthority

LOG = logging.getLogger(__name__)


class RequirementsParser(object):
    _inner = None

    class Inner(object):
        _rbac_map = None

        def __init__(self, filepath):
            with open(filepath) as f:
                RequirementsParser.Inner._rbac_map = \
                    list(yaml.safe_load_all(f))

    def __init__(self, filepath):
        if RequirementsParser._inner is None:
            RequirementsParser._inner = RequirementsParser.Inner(filepath)

    @staticmethod
    def parse(component):
        try:
            for section in RequirementsParser.Inner._rbac_map:
                if component in section:
                    return section[component]
        except yaml.parser.ParserError:
            LOG.error("Error while parsing the requirements YAML file. Did "
                      "you pass a valid component name from the test case?")
        return None


class RequirementsAuthority(RbacAuthority):
    def __init__(self, filepath=None, component=None):
        if filepath is not None and component is not None:
            self.roles_dict = RequirementsParser(filepath).parse(component)
        else:
            self.roles_dict = None

    def allowed(self, rule_name, role):
        if self.roles_dict is None:
            raise exceptions.InvalidConfiguration(
                "Roles dictionary parsed from requirements YAML file is "
                "empty. Ensure the requirements YAML file is correctly "
                "formatted.")
        try:
            _api = self.roles_dict[rule_name]
            return role in _api
        except KeyError:
            raise KeyError("'%s' API is not defined in the requirements YAML "
                           "file" % rule_name)
        return False



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.identity import rbac_base


class IdentityUserV3RbacTest(rbac_base.BaseIdentityV3RbacTest):

    @classmethod
    def resource_setup(cls):
        super(IdentityUserV3RbacTest, cls).resource_setup()
        cls.default_user_id = cls.os_primary.credentials.user_id

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:create_user")
    @decorators.idempotent_id('0f148510-63bf-11e6-4522-080044d0d904')
    def test_create_user(self):
        with self.rbac_utils.override_role(self):
            self.setup_test_user()

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:update_user")
    @decorators.idempotent_id('0f148510-63bf-11e6-4522-080044d0d905')
    def test_update_user(self):
        user = self.setup_test_user()
        new_email = data_utils.rand_name(
            self.__class__.__name__ + '-user_email')

        with self.rbac_utils.override_role(self):
            self.users_client.update_user(user['id'],
                                          name=user['name'],
                                          email=new_email)

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:delete_user")
    @decorators.idempotent_id('0f148510-63bf-11e6-4522-080044d0d906')
    def test_delete_user(self):
        user = self.setup_test_user()

        with self.rbac_utils.override_role(self):
            self.users_client.delete_user(user['id'])

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:list_users")
    @decorators.idempotent_id('0f148510-63bf-11e6-4522-080044d0d907')
    def test_list_users(self):
        with self.rbac_utils.override_role(self):
            self.users_client.list_users()

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:get_user")
    @decorators.idempotent_id('0f148510-63bf-11e6-4522-080044d0d908')
    def test_show_own_user(self):
        with self.rbac_utils.override_role(self):
            self.users_client.show_user(self.default_user_id)

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:list_groups_for_user")
    @decorators.idempotent_id('bd5946d4-46d2-423d-a800-a3e7aabc18b3')
    def test_list_own_user_group(self):
        with self.rbac_utils.override_role(self):
            self.users_client.list_user_groups(self.default_user_id)

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:list_user_projects")
    @decorators.idempotent_id('0f148510-63bf-11e6-1564-080044d0d909')
    def test_list_own_user_projects(self):
        with self.rbac_utils.override_role(self):
            self.users_client.list_user_projects(self.default_user_id)



# Copyright 2017 AT&T Corporation.
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base as vol_base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils

from patrole_tempest_plugin import rbac_utils

CONF = config.CONF


class BaseVolumeRbacTest(rbac_utils.RbacUtilsMixin,
                         vol_base.BaseVolumeTest):
    # NOTE(felipemonteiro): Patrole currently only tests the v3 Cinder API
    # because it is the current API and because policy enforcement does not
    # change between API major versions. So, it is not necessary to specify
    # the `_api_version` in any test class. However, specify microversions in
    # subclasses if necessary.
    _api_version = 3

    @classmethod
    def skip_checks(cls):
        super(BaseVolumeRbacTest, cls).skip_checks()
        cls.skip_rbac_checks()

    @classmethod
    def setup_clients(cls):
        super(BaseVolumeRbacTest, cls).setup_clients()
        cls.setup_rbac_utils()
        cls.volume_hosts_client = cls.os_primary.volume_hosts_v2_client
        cls.volume_types_client = cls.os_primary.volume_types_v2_client
        cls.groups_client = cls.os_primary.groups_v3_client
        cls.group_types_client = cls.os_primary.group_types_v3_client

    @classmethod
    def create_volume_type(cls, name=None, **kwargs):
        """Create a test volume-type"""
        name = name or data_utils.rand_name(cls.__name__ + '-volume-type')
        volume_type = cls.volume_types_client.create_volume_type(
            name=name, **kwargs)['volume_type']
        cls.addClassResourceCleanup(
            cls.volume_types_client.wait_for_resource_deletion,
            volume_type['id'])
        cls.addClassResourceCleanup(
            test_utils.call_and_ignore_notfound_exc,
            cls.volume_types_client.delete_volume_type, volume_type['id'])
        return volume_type

    def create_group_type(self, name=None, ignore_notfound=False, **kwargs):
        """Create a test group-type"""
        name = name or data_utils.rand_name(
            self.__class__.__name__ + '-group-type')
        group_type = self.group_types_client.create_group_type(
            name=name, **kwargs)['group_type']

        if ignore_notfound:
            self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                            self.group_types_client.delete_group_type,
                            group_type['id'])
        else:
            self.addCleanup(self.group_types_client.delete_group_type,
                            group_type['id'])

        return group_type



#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base


class SecurtiyGroupsRbacTest(rbac_base.BaseV2ComputeRbacTest):
    """Tests non-deprecated security group policies. Requires network service.

    This class tests non-deprecated policies for adding and removing a security
    group to and from a server.
    """

    @classmethod
    def skip_checks(cls):
        super(SecurtiyGroupsRbacTest, cls).skip_checks()
        # All the tests below require the network service.
        # NOTE(gmann) Currently 'network' service is always True in
        # utils.get_service_list() So below check is not much of use.
        # Commenting the below check as Tempest is moving the get_service_list
        # from test.py to utils.
        # If we want to check 'network' service availability, then
        # get_service_list can be used from new location.
        # if not utils.get_service_list()['network']:
        #    raise cls.skipException(
        #        'Skipped because the network service is not available')

    @classmethod
    def setup_credentials(cls):
        # A network and a subnet will be created for these tests.
        cls.set_network_resources(network=True, subnet=True)
        super(SecurtiyGroupsRbacTest, cls).setup_credentials()

    @classmethod
    def resource_setup(cls):
        super(SecurtiyGroupsRbacTest, cls).resource_setup()
        cls.server = cls.create_test_server(wait_until='ACTIVE')

    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-security-groups")
    @decorators.idempotent_id('3db159c6-a467-469f-9a25-574197885520')
    def test_list_security_groups_by_server(self):
        with self.rbac_utils.override_role(self):
            self.servers_client.list_security_groups_by_server(
                self.server['id'])

    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-security-groups")
    @decorators.idempotent_id('ea1ca73f-2d1d-43cb-9a46-900d7927b357')
    def test_create_security_group_for_server(self):
        sg_name = self.create_security_group()['name']

        with self.rbac_utils.override_role(self):
            self.servers_client.add_security_group(self.server['id'],
                                                   name=sg_name)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.servers_client.remove_security_group,
                        self.server['id'], name=sg_name)

    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-security-groups")
    @decorators.idempotent_id('0ad2e856-e2d3-4ac5-a620-f93d0d3d2626')
    def test_remove_security_group_from_server(self):
        sg_name = self.create_security_group()['name']

        self.servers_client.add_security_group(self.server['id'], name=sg_name)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.servers_client.remove_security_group,
                        self.server['id'], name=sg_name)

        with self.rbac_utils.override_role(self):
            self.servers_client.remove_security_group(
                self.server['id'], name=sg_name)


class SecurityGroupsRbacMaxV235Test(rbac_base.BaseV2ComputeRbacTest):

    # Tests in this class will fail with a 404 from microversion 2.36,
    # according to:
    # https://developer.openstack.org/api-ref/compute/#security-groups-os-security-groups-deprecated
    max_microversion = '2.35'

    @classmethod
    def skip_checks(cls):
        super(SecurityGroupsRbacMaxV235Test, cls).skip_checks()
        # All the tests below require the network service.
        # NOTE(gmann) Currently 'network' service is always True in
        # utils.get_service_list() So below check is not much of use.
        # Commenting the below check as Tempest is moving the get_service_list
        # from test.py to utils.
        # If we want to check 'network' service availability, then
        # get_service_list can be used from new location.
        # if not utils.get_service_list()['network']:
        #    raise cls.skipException(
        #        'Skipped because the network service is not available')

    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-security-groups")
    @decorators.idempotent_id('4ac58e49-48c1-4fca-a6c3-3f95fb99eb77')
    def test_list_security_groups(self):
        with self.rbac_utils.override_role(self):
            self.security_groups_client.list_security_groups()

    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-security-groups")
    @decorators.idempotent_id('e8fe7f5a-69ee-412d-81d3-a8c7a488b54d')
    def test_create_security_groups(self):
        with self.rbac_utils.override_role(self):
            self.create_security_group()['id']

    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-security-groups")
    @decorators.idempotent_id('59127e8e-302d-11e7-93ae-92361f002671')
    def test_delete_security_groups(self):
        sec_group_id = self.create_security_group()['id']
        with self.rbac_utils.override_role(self):
            self.security_groups_client.delete_security_group(sec_group_id)

    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-security-groups")
    @decorators.idempotent_id('3de5c6bc-b822-469e-a627-82427d38b067')
    def test_update_security_groups(self):
        sec_group_id = self.create_security_group()['id']
        new_name = data_utils.rand_name()
        new_desc = data_utils.rand_name()

        with self.rbac_utils.override_role(self):
            self.security_groups_client.update_security_group(
                sec_group_id, name=new_name, description=new_desc)

    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-security-groups")
    @decorators.idempotent_id('6edc0320-302d-11e7-93ae-92361f002671')
    def test_show_security_groups(self):
        sec_group_id = self.create_security_group()['id']
        with self.rbac_utils.override_role(self):
            self.security_groups_client.show_security_group(sec_group_id)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.volume import rbac_base


class EncryptionTypesV3RbacTest(rbac_base.BaseVolumeRbacTest):

    @classmethod
    def skip_checks(cls):
        super(EncryptionTypesV3RbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('encryption', 'volume'):
            msg = "%s skipped as encryption not enabled." % cls.__name__
            raise cls.skipException(msg)

    @classmethod
    def setup_clients(cls):
        super(EncryptionTypesV3RbacTest, cls).setup_clients()
        cls.encryption_types_client = cls.os_primary.encryption_types_v2_client

    def _create_volume_type_encryption(self):
        vol_type_id = self.create_volume_type()['id']
        self.encryption_types_client.create_encryption_type(
            vol_type_id,
            provider="nova.volume.encryptors.luks.LuksEncryptor",
            control_location="front-end")['encryption']
        return vol_type_id

    @decorators.idempotent_id('ffd94ce5-c24b-4b6c-84c9-c5aad8c3010c')
    @rbac_rule_validation.action(
        service="cinder",
        rule="volume_extension:volume_type_encryption")
    def test_create_volume_type_encryption(self):
        vol_type_id = self.create_volume_type()['id']
        with self.rbac_utils.override_role(self):
            self.encryption_types_client.create_encryption_type(
                vol_type_id,
                provider="nova.volume.encryptors.luks.LuksEncryptor",
                control_location="front-end")['encryption']

    @decorators.idempotent_id('6599e72e-acef-4c0d-a9b2-463fca30d1da')
    @rbac_rule_validation.action(
        service="cinder",
        rule="volume_extension:volume_type_encryption")
    def test_delete_volume_type_encryption(self):
        vol_type_id = self._create_volume_type_encryption()
        with self.rbac_utils.override_role(self):
            self.encryption_types_client.delete_encryption_type(vol_type_id)

    @decorators.idempotent_id('42da9fec-32fd-4dca-9242-8a53b2fed25a')
    @rbac_rule_validation.action(
        service="cinder",
        rule="volume_extension:volume_type_encryption")
    def test_update_volume_type_encryption(self):
        vol_type_id = self._create_volume_type_encryption()
        with self.rbac_utils.override_role(self):
            self.encryption_types_client.update_encryption_type(
                vol_type_id,
                control_location="front-end")

    @decorators.idempotent_id('1381a3dc-248f-4282-b231-c9399018c804')
    @rbac_rule_validation.action(
        service="cinder",
        rule="volume_extension:volume_type_encryption")
    def test_show_volume_type_encryption(self):
        vol_type_id = self._create_volume_type_encryption()
        with self.rbac_utils.override_role(self):
            self.encryption_types_client.show_encryption_type(vol_type_id)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log

from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions

from patrole_tempest_plugin import rbac_exceptions
from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base as base

CONF = config.CONF
LOG = log.getLogger(__name__)


class ComputeServersRbacTest(base.BaseV2ComputeRbacTest):

    @classmethod
    def setup_clients(cls):
        super(ComputeServersRbacTest, cls).setup_clients()
        cls.networks_client = cls.os_primary.networks_client
        cls.ports_client = cls.os_primary.ports_client
        cls.subnets_client = cls.os_primary.subnets_client

    @classmethod
    def resource_setup(cls):
        super(ComputeServersRbacTest, cls).resource_setup()
        cls.server = cls.create_test_server(wait_until='ACTIVE')

    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:servers:create")
    @decorators.idempotent_id('4f34c73a-6ddc-4677-976f-71320fa855bd')
    def test_create_server(self):
        with self.rbac_utils.override_role(self):
            server = self.servers_client.create_server(
                name=data_utils.rand_name(self.__class__.__name__ + '-Server'),
                flavorRef=CONF.compute.flavor_ref,
                imageRef=CONF.compute.image_ref)['server']
        self.addCleanup(waiters.wait_for_server_termination,
                        self.servers_client, server['id'])
        self.addCleanup(self.servers_client.delete_server, server['id'])

    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:servers:create:forced_host")
    @decorators.idempotent_id('0ae3c401-52ab-41bc-ab96-c598a65d9ae5')
    def test_create_server_forced_host(self):
        # Retrieve 'nova' zone host information from availiability_zone_list
        zones = self.availability_zone_client.list_availability_zones(
            detail=True)['availabilityZoneInfo']
        hosts = [zone['hosts'] for zone in zones if zone['zoneName'] == 'nova']

        # We just need any host out of the hosts list to build the
        # availability_zone attribute. So, picking the first one is fine.
        # The first key of the dictionary specifies the host name.
        host = list(hosts[0].keys())[0]
        availability_zone = 'nova:' + host

        with self.rbac_utils.override_role(self):
            server = self.servers_client.create_server(
                name=data_utils.rand_name(self.__class__.__name__ + '-Server'),
                flavorRef=CONF.compute.flavor_ref,
                imageRef=CONF.compute.image_ref,
                availability_zone=availability_zone)['server']
        self.addCleanup(waiters.wait_for_server_termination,
                        self.servers_client, server['id'])
        self.addCleanup(self.servers_client.delete_server, server['id'])

    @utils.services('volume')
    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:servers:create:attach_volume")
    @decorators.idempotent_id('eeddac5e-15aa-454f-838d-db608aae4dd8')
    def test_create_server_attach_volume(self):
        # To create a bootable volume, the UUID of the image from which
        # to create the volume must be included as the imageRef attribute in
        # the request body.
        volume_id = self.create_volume(
            imageRef=CONF.compute.image_ref,
            size=CONF.volume.volume_size)['id']

        bd_map_v2 = [{'uuid': volume_id,
                      'source_type': 'volume',
                      'destination_type': 'volume',
                      'boot_index': 0,
                      'delete_on_termination': False}]
        device_mapping = {'block_device_mapping_v2': bd_map_v2}

        with self.rbac_utils.override_role(self):
            # Use image_id='' to avoid using the default image in tempest.conf.
            server = self.servers_client.create_server(
                name=data_utils.rand_name(self.__class__.__name__ + '-Server'),
                flavorRef=CONF.compute.flavor_ref,
                imageRef='',
                **device_mapping)['server']
        waiters.wait_for_server_status(
            self.servers_client, server['id'], 'ACTIVE')
        # Delete the server and wait for the volume to become available to
        # avoid clean up errors.
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        waiters.wait_for_volume_resource_status,
                        self.volumes_client, volume_id, 'available')
        self.addCleanup(waiters.wait_for_server_termination,
                        self.servers_client, server['id'])
        self.addCleanup(self.delete_server, server['id'])

    @utils.services('network')
    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:servers:create:attach_network")
    @decorators.idempotent_id('b44cd4ff-50a4-42ce-ada3-724e213cd540')
    def test_create_server_attach_network(self):
        def _create_network_resources():
            # Create network
            network_name = data_utils.rand_name(
                self.__class__.__name__ + '-network')

            network = self.networks_client.create_network(
                name=network_name, port_security_enabled=True)['network']
            self.addCleanup(self.networks_client.delete_network, network['id'])

            # Create subnet for the network
            subnet_name = data_utils.rand_name(
                self.__class__.__name__ + '-subnet')
            subnet = self.subnets_client.create_subnet(
                name=subnet_name,
                network_id=network['id'],
                cidr=CONF.network.project_network_cidr,
                ip_version=4)['subnet']
            self.addCleanup(self.subnets_client.delete_subnet, subnet['id'])

            return network

        network = _create_network_resources()
        network_id = {'uuid': network['id']}

        with self.rbac_utils.override_role(self):
            server = self.servers_client.create_server(
                name=data_utils.rand_name(self.__class__.__name__ + '-Server'),
                flavorRef=CONF.compute.flavor_ref,
                imageRef=CONF.compute.image_ref,
                networks=[network_id])['server']
        self.addCleanup(waiters.wait_for_server_termination,
                        self.servers_client, server['id'])
        self.addCleanup(self.servers_client.delete_server, server['id'])

    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:servers:delete")
    @decorators.idempotent_id('062e3440-e873-4b41-9317-bf6d8be50c12')
    def test_delete_server(self):
        server = self.create_test_server(wait_until='ACTIVE')

        with self.rbac_utils.override_role(self):
            self.servers_client.delete_server(server['id'])
        waiters.wait_for_server_termination(
            self.servers_client, server['id'])

    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:servers:update")
    @decorators.idempotent_id('077b17cb-5621-43b9-8adf-5725f0d7a863')
    def test_update_server(self):
        new_name = data_utils.rand_name(self.__class__.__name__ + '-server')
        with self.rbac_utils.override_role(self):
            try:
                self.servers_client.update_server(self.server['id'],
                                                  name=new_name)
                waiters.wait_for_server_status(self.servers_client,
                                               self.server['id'], 'ACTIVE')
            except exceptions.ServerFault as e:
                # Some other policy may have blocked it.
                LOG.info("ServerFault exception caught. Some other policy "
                         "blocked updating of server")
                raise rbac_exceptions.RbacConflictingPolicies(e)
